Package: geneticae
Title: Statistical Tools for the Analysis of Multi Environment
        Agronomic Trials
Version: 1.0.1
Authors@R: c(
  person(given = "Julia",
         family = "Angelini",
         role = c("aut", "cre"),
         email = "jangelini_93@hotmail.com",
         comment = c(ORCID = "0000-0002-5815-1771")
         ),
  person(given = "Marcos",
         family = "Prunello",
         role = c("aut"),
         email = "marcosprunello@gmail.com",
         comment = c(ORCID = "0000-0002-9611-527X")
         ),
  person(given = "Gerardo",
         family = "Cervigni",
         role = c("aut")
         )
  )
Description: Provides tools for the analysis of multi-environment agronomic 
    trials, with a specific focus on plant breeding experiments. 
    Implements the Additive Main effects and Multiplicative Interaction 
    (AMMI) model (Gauch, 1992, ISBN:9780444892409) and the Site Regression 
    (SREG) model (Cornelius, 1996, <doi:10.1201/9780367802226>). To ensure 
    reliable results even with outliers or missing data, it includes robust 
    versions of AMMI (Rodrigues et al., 2016, 
    <doi:10.1093/bioinformatics/btv533>) and SREG (Angelini et al., 2022, 
    <doi:10.1080/15427528.2022.2051217>). Furthermore, the package offers 
    advanced imputation techniques for multi-environment data, covering 
    classical methodologies (Arciniegas-Alarcón et al., 2014, 
    <doi:10.2478/bile-2014-0006>) and recently published imputation methods 
    for MET data (Angelini et al., 2024, <doi:10.1007/s10681-024-03344-z>).
License: GPL-2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Imports: stats, ggforce, ggplot2, scales, MASS, pcaMethods, rrcov,
        dplyr, missMDA, tidyr, rlang, corpcor
Suggests: agridat, spelling, knitr, rmarkdown, patchwork, testthat
VignetteBuilder: knitr
Depends: R (>= 2.12.0)
URL: https://jangelini.github.io/geneticae/,
        https://github.com/jangelini/geneticae
BugReports: https://github.com/jangelini/geneticae/issues
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-20 22:45:40 UTC; julia
Author: Julia Angelini [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5815-1771>),
  Marcos Prunello [aut] (ORCID: <https://orcid.org/0000-0002-9611-527X>),
  Gerardo Cervigni [aut]
Maintainer: Julia Angelini <jangelini_93@hotmail.com>
Repository: CRAN
Date/Publication: 2026-04-21 10:52:14 UTC
Built: R 4.5.2; ; 2026-04-21 19:21:24 UTC; unix

Package: geneticae
Title: Statistical Tools for the Analysis of Multi Environment
        Agronomic Trials
Version: 1.0.1
Authors@R: c(
  person(given = "Julia",
         family = "Angelini",
         role = c("aut", "cre"),
         email = "jangelini_93@hotmail.com",
         comment = c(ORCID = "0000-0002-5815-1771")
         ),
  person(given = "Marcos",
         family = "Prunello",
         role = c("aut"),
         email = "marcosprunello@gmail.com",
         comment = c(ORCID = "0000-0002-9611-527X")
         ),
  person(given = "Gerardo",
         family = "Cervigni",
         role = c("aut")
         )
  )
Description: Provides tools for the analysis of multi-environment agronomic 
    trials, with a specific focus on plant breeding experiments. 
    Implements the Additive Main effects and Multiplicative Interaction 
    (AMMI) model (Gauch, 1992, ISBN:9780444892409) and the Site Regression 
    (SREG) model (Cornelius, 1996, <doi:10.1201/9780367802226>). To ensure 
    reliable results even with outliers or missing data, it includes robust 
    versions of AMMI (Rodrigues et al., 2016, 
    <doi:10.1093/bioinformatics/btv533>) and SREG (Angelini et al., 2022, 
    <doi:10.1080/15427528.2022.2051217>). Furthermore, the package offers 
    advanced imputation techniques for multi-environment data, covering 
    classical methodologies (Arciniegas-Alarcón et al., 2014, 
    <doi:10.2478/bile-2014-0006>) and recently published imputation methods 
    for MET data (Angelini et al., 2024, <doi:10.1007/s10681-024-03344-z>).
License: GPL-2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Imports: stats, ggforce, ggplot2, scales, MASS, pcaMethods, rrcov,
        dplyr, missMDA, tidyr, rlang, corpcor
Suggests: agridat, spelling, knitr, rmarkdown, patchwork, testthat
VignetteBuilder: knitr
Depends: R (>= 2.12.0)
URL: https://jangelini.github.io/geneticae/,
        https://github.com/jangelini/geneticae
BugReports: https://github.com/jangelini/geneticae/issues
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-20 22:45:40 UTC; julia
Author: Julia Angelini [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5815-1771>),
  Marcos Prunello [aut] (ORCID: <https://orcid.org/0000-0002-9611-527X>),
  Gerardo Cervigni [aut]
Maintainer: Julia Angelini <jangelini_93@hotmail.com>
Repository: CRAN
Date/Publication: 2026-04-21 10:52:14 UTC
Built: R 4.6.0; ; 2026-05-01 01:45:47 UTC; unix

# Generated by roxygen2: do not edit by hand

export(BiplotImputfun)
export(EM.SREG)
export(imputation)
export(rAMMIModel)
export(rAMMIPlot)
export(rSREGModel)
export(rSREGPlot)
import(ggplot2)
importFrom(MASS,ginv)
importFrom(MASS,rlm)
importFrom(corpcor,fast.svd)
importFrom(dplyr,"%>%")
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,group_by)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(ggforce,geom_arc)
importFrom(ggforce,geom_circle)
importFrom(ggplot2,aes)
importFrom(ggplot2,arrow)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer_scales)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,chull)
importFrom(grid,unit)
importFrom(missMDA,imputePCA)
importFrom(pcaMethods,loadings)
importFrom(pcaMethods,pca)
importFrom(pcaMethods,robustSvd)
importFrom(pcaMethods,scores)
importFrom(rlang,sym)
importFrom(rrcov,CovControlMrcd)
importFrom(rrcov,PcaCov)
importFrom(rrcov,PcaGrid)
importFrom(rrcov,PcaHubert)
importFrom(rrcov,PcaLocantore)
importFrom(rrcov,PcaProj)
importFrom(rrcov,getEigenvalues)
importFrom(rrcov,getLoadings)
importFrom(rrcov,getScores)
importFrom(scales,alpha)
importFrom(scales,squish)
importFrom(stats,as.formula)
importFrom(stats,lm)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(tidyr,pivot_wider)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# geneticae <img src="man/figures/baseplot.png" align="right" alt="" width="160"/>

## Statistical Tools for the Analysis of Multi Environment Agronomic Trials

Web: <https://jangelini.github.io/geneticae/>

CRAN: <https://CRAN.R-project.org/package=geneticae/index.html>

<!-- badges: start -->

<!-- dejo esto porque todavia hay links que no estan disponibles: -->

[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)

<!-- cuando este en cran agregar estas -->

[![CRAN
status](https://www.r-pkg.org/badges/version/geneticae)](https://CRAN.R-project.org/package=geneticae)
[![Downloads](https://cranlogs.r-pkg.org/badges/geneticae?color=blue)](https://CRAN.R-project.org/package=geneticae)
<!-- [![Codecov test coverage](https://codecov.io/gh/r-lib/geneticae/branch/master/graphs/badge.svg)](https://codecov.io/gh/r-lib/geneticae?branch=master) -->
<!-- badges: end -->

Understanding the relationship between crops performance and environment
is a key problem for plant breeders and geneticists. In advanced stages
of breeding programs, in which few genotypes are evaluated,
multi-environmental trials (MET) are one of the most used experiments.
Such studies test a number of genotypes in multiple environments in
order to identify the superior genotypes according to their performance.
In these experimentes, crop performance is modeled as a function of
genotype (G), environment (E) and genotype-environment interaction
(GEI). The presence of GEI generates differential genotypic responses in
the different environments. Therefore appropriate statistical methods
should be used to obtain an adequate GEI analysis, which is essential
for plant breeders.

The average performance of genotypes through different environments can
only be considered in the absence of GEI. However, GEI is almost always
present and the comparison of the mean performance between genotypes is
not enough. The most widely used methods to analyze MET data are based
on regression models, analysis of variance (ANOVA) and multivariate
techniques. In particular, two statistical models are widely used among
plant breeders as they provide useful graphical tools for the study of
GEI: the Additive Main effects and Multiplicative Interaction model
(AMMI) and the Site Regression Model (SREG). However, these models are
not always efficient enough to analyze MET data structure of plant
breeding programs. They present serious limitations in the presence of
atypical observations and missing values, which occur very frequently.
To overcome this, several imputation alternatives (Angelini et al.,
2024) and a robust AMMI and a SREG model were recently proposed in
literature (Rodrigues et al., 2016; Angelini et al., 2022).

The `geneticae` package was created to gather in one place the most
useful functions for this type of analysis and it also implements new
methodology which can be found in recent literature. More importantly,
`geneticae` is the first package to implement the robust AMMI models
proposed by Rodrigues et al. (2016), the robusts SREG proposed by
Angelini et al. (2022) and new imputation methods proposed by Angelini
et al. (2024). In addition, there is no need to preprocess the data to
use the `geneticae` package, as it the case of some previous packages
which require a data frame or matrix containing genotype by environment
means with the genotypes in rows and the environments in columns. In
this package, data in long format is required. There is no restriction
on columns names of genotypes, environments, repetitions (if any) and
phenotypic traits of interest. Also, extra information that will not be
used in the analysis may be present in the dataset. Finally, `geneticae`
offers a wide variety of options to customize the biplots, which are
part of the graphical output of these methods.

This package can be used through
[this](https://geneticae.shinyapps.io/geneticae-shiny-web-app/) Shiny
app, making it available not only for R programmers.

## Installation

You can install the released version of `geneticae` from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("geneticae")
```

You can install the development version from our [GitHub
repo](https://github.com/jangelini/geneticae) with:

``` r
# install.packages("devtools")
devtools::install_github("jangelini/geneticae")
```

## Shiny app

You can use `geneticae` through
[this](https://geneticae.shinyapps.io/geneticae-shiny-web-app/) Shiny
app. Source code is in [GitHub
repo](https://github.com/jangelini/Geneticae-Shiny-Web-APP).

## Getting Started

If you are just getting started with `geneticae` we recommend visiting
the vignettes and exploring the examples throughout the package
documentation.

Here we present a small example.

The dataset `yan.winterwheat` has information about the yield of 18
winter wheat varieties grown in nine environments in Ontario at 1993.
The function `GGEPlot()` builds several GGE biplots views. The basic
biplot is produced with the argument `type="Biplot"`. If the function is
used with the argument `type = "Selected Environment"` and the name of
one environment is provided in `selectedE`, a line that passes through
the environment marker (i.e. OA93), and the biplot origin is added. The
most suitable cultivars for that particular environment can be
identified looking at their projection onto this axis. Thus, at the
environment OA93, the highest-yielding cultivar was *Zav*, and the
lowest-yielding cultivar was *Luc*. The perpendicular line to the OA93
axis separates genotypes that yielded above and below the mean in this
environment.

``` r
library(geneticae)
library(agridat)
data(yan.winterwheat)


GGE1 <- rSREGModel(yan.winterwheat, genotype = "gen", environment = "env", 
                   response = "yield",  model = "SREG")

rSREGPlot(GGE1, type = "Selected Environment", selectedE = "OA93", footnote = FALSE, titles = FALSE)
```

<div class="figure" style="text-align: center">

<img src="man/figures/README-unnamed-chunk-4-1.png" alt="Figure: comparison of cultivar performance in a selected environment." width="45%" />
<p class="caption">

Figure: comparison of cultivar performance in a selected environment.
</p>

</div>

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(warning = F, collapse = TRUE, message = F, out.width = "60%")

## ----echo=F, fig.align='center', fig.cap='Figure: Polygon view of the GGE biplot, showing which cultivars presented highest yield in each environment. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'----
library(geneticae)
library(agridat)
data(yan.winterwheat)
GGE1 <- rSREGModel(yan.winterwheat, genotype = "gen", environment = "env", 
                 response = "yield")

rSREGPlot(GGE1, type = "Which Won Where/What", footnote = F,
        titles = F, axis_expand = 1.5)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(warning = F, message = F, out.width = "60%")
library(dplyr)

## ----eval=F-------------------------------------------------------------------
# install.packages("geneticae")

## ----eval=F-------------------------------------------------------------------
# # install.packages("devtools")
# devtools::install_github("jangelini/geneticae")

## -----------------------------------------------------------------------------
library(geneticae)

## -----------------------------------------------------------------------------
library(agridat)
data(yan.winterwheat)
head(yan.winterwheat)

## -----------------------------------------------------------------------------
data(plrv)
head(plrv)

## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 1: GE biplot based on yield data of 1993 Ontario winter wheat performance trials.  The 71.66% of GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'----

rAMMI_clasic <- rAMMIModel(yan.winterwheat, genotype = "gen", environment = "env", 
      response = "yield", type = "AMMI")

rAMMIPlot(rAMMI_clasic, titles = TRUE, footnote = TRUE)


## -----------------------------------------------------------------------------
GGE1 <- rSREGModel(yan.winterwheat, genotype = "gen", environment = "env", 
                 response = "yield")

## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 2: GGE biplot based on yield data of 1993 Ontario winter wheat performance trials. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'----
rSREGPlot(GGE1, type = "Biplot", footnote = F, titles = F)

## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 3: comparison of cultivar performance in a selected environment (OA93). The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms.'----
rSREGPlot(GGE1, type = "Selected Environment", selectedE = "OA93", 
        footnote = F, titles = F)

## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 4: comparison of the performance of cultivar Luc in different environments. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. '----
rSREGPlot(GGE1, type = "Selected Genotype", selectedG = "Kat", 
        footnote = F, titles = F)

## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 5: comparison of the cultivars _Kat_ and _Cas_. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'----
rSREGPlot(GGE1, type = "Comparison of Genotype", 
        selectedG1 = "Kat", selectedG2 = "Cas", 
        footnote = F, titles = F, axis_expand = 1.5)

## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 6: polygon view of the GGE biplot, showing which cultivars presented highest yield in each environment. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'----
rSREGPlot(GGE1, type = "Which Won Where/What", footnote = F,
        titles = F, axis_expand = 1.5)


## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 7: average environment view of the GGE biplot based on genotype-focused scaling, showing mean yield and stability of genotypes. '----
data <- yan.winterwheat[yan.winterwheat$env %in% c("BH93", "EA93","HW93", "ID93",
                                                   "NN93", "RN93", "WP93"), ]
data <- droplevels(data)
GGE2 <- rSREGModel(data, genotype = "gen", environment = "env", 
                 response = "yield", SVP = "row")

rSREGPlot(GGE2, type = "Mean vs. Stability", footnote = F, titles = F, sizeEnv = 0)


## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 8: Classification of genotypes with respect to the ideal genotype. Genotype-focused scaling is used.', warning=FALSE----
rSREGPlot(GGE2, type = "Ranking Genotypes", footnote = F, titles = F, sizeEnv = 0)

## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 9: Relationship between environments. Environment-focused scaling is used.'----

GGE3 <- rSREGModel(data, genotype = "gen", environment = "env", 
                 response = "yield", SVP = "column")
rSREGPlot(GGE3, type = "Relationship Among Environments", footnote = F, titles = F)

## ----fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 10: classification of environments with respect to the ideal environment. Environment-focused scaling is used.'----
rSREGPlot(GGE3, type = "Ranking Environments", footnote = F, titles = F)

## ----fig.width=12, fig.height=10, fig.align='center', dpi=300, out.width="100%", fig.cap='Figure 11: GGE biplot based on the height of 25 progenies of Eucalyptus grandis from trials carried out in seven environments in the southern and southeastern regions of Brazil. The scaling method used is symmetrical singular value partitioning. SREG (A), hSREG (B), CovSREG (C), and ppSREG (D).'----

library(patchwork)
library(ggplot2)
data("lavoranti.eucalyptus")

# Bellthorpe
lavoranti_Bellthorpe <- droplevels(subset(lavoranti.eucalyptus, origin=="Bellthorpe"))

Nenv <-length(levels(lavoranti_Bellthorpe$loc))
Ngen <- length(levels(lavoranti_Bellthorpe$gen))

lavoranti_Bellthorpe$genot <- rep(1:Ngen, Nenv)
lavoranti_Bellthorpe$genot <- as.factor(lavoranti_Bellthorpe$genot)


# SREG Classic
SREG_classic <- rSREGModel(lavoranti_Bellthorpe, genotype = "genot", 
                          environment = "loc", response = "height", 
                          model = "SREG")

# hSREG
hSREG <- rSREGModel(lavoranti_Bellthorpe, genotype = "genot", 
                    environment = "loc", response = "height", 
                    model = "hSREG")

# CovSREG
CovSREG <- rSREGModel(lavoranti_Bellthorpe, genotype = "genot", 
                    environment = "loc", response = "height", 
                    model = "hSREG")



# ppSREG
ppSREG <- rSREGModel(lavoranti_Bellthorpe, genotype = "genot", 
                    environment = "loc", response = "height", 
                    model = "ppSREG")

# Graphs
p1 <- rSREGPlot(SREG_classic, footnote = F, titles = T)
p2 <- hSREG_plot  <- rSREGPlot(hSREG, footnote = F, titles = T)
p3 <- CovSREG_plot <- rSREGPlot(CovSREG, footnote = F, titles = T)
p4 <- ppSREG_plot  <- rSREGPlot(ppSREG, footnote = F, titles = T)

grafico_robusto_SREG <- (p1 + p2) / (p3 + p4) + 
  plot_annotation(tag_levels = 'A') & 
  theme(plot.tag = element_text(size = 18, face = "bold"))

grafico_robusto_SREG

## -----------------------------------------------------------------------------
# Generating missing data
yan.winterwheat[1,3] <- NA
yan.winterwheat[3,3] <- NA
yan.winterwheat[2,3] <- NA

## -----------------------------------------------------------------------------
imputation(yan.winterwheat, genotype = "gen", environment = "env", 
           response = "yield", type = "EM-SREG")


imputation(yan.winterwheat, genotype = "gen", environment = "env", 
           response = "yield", type = "EM-bSREG")

## ----echo = FALSE-------------------------------------------------------------
sessionInfo()

if(requireNamespace('spelling', quietly = TRUE))
  spelling::spell_check_test(vignettes = "/media/julia/KINGSTON/Tesis doctorado en estadistica/Actualizacion geneticae/geneticae/", error = FALSE,
                             skip_on_cran = TRUE)

#'EM-AMMI imputation method
#'
#'@description This function was written by Paderewski (2013) and allow imputing
#'  missing values by the EM-AMMI algorithm
#'
#'@param X a data frame or matrix with genotypes in rows and
#'  environments in columns when there are no replications of the
#'  experiment.
#'@param PC.nb the number of principal components in the AMMI model that will be
#'  used; default value is 1. For PC.nb=0 only main effects are used to
#'  estimate cells in the data table (the interaction is ignored). The number of
#'  principal components must not be greater than min(number of rows in the X
#'  table, number of columns in the X table)–2.
#'@param initial.values (optional) initial values of missing cells. It
#'  can be a single value, which then will be used for all empty cells, or a
#'  vector of length equal to the number of missing cells (starting from the
#'  missing values in the first column). If omitted, initial values will be
#'  obtained by the main effects from the corresponding model, that is, by
#'  grand mean of observed data increased (or decreased) by row and column
#'  main effects.
#'@param precision (optional) algorithm converges if the maximal change in
#'  the values of the missing cells in two subsequent steps is not greater than
#'  this value (the default is 0.01).
#'@param max.iter (optional) a maximum permissible number of iterations (that
#'  is, number of repeats of the algorithm’s steps 2 through 5); default
#'  value is 1000.
#'@param change.factor (optional) introduced by analogy to step size in gradient
#'  descent method, this parameter that can shorten the time of executing the
#'  algorithm by decreasing the number of iterations. The change.factor=1
#'  (default) defines that the previous approximation is changed with the new
#'  values of missing cells (standard EM-AMMI algorithm). However, when
#'  change.factor<1, then the new approximations are computed and the values of
#'  missing cells are changed in the direction of this new approximation but the
#'  change is smaller. It could be useful if the changes are cyclic and thus
#'  convergence could not be reached. Usually, this argument should not affect
#'  the final outcome (that is, the imputed values) as compared to the default
#'  value of change.factor=1.
#'@param simplified.model the AMMI model contains the general mean, effects of
#'  rows, columns and interaction terms. So the EM-AMMI algorithm in step 2
#'  calculates the current effects of rows and columns; these effects change
#'  from iteration to iteration because the empty (at the outset) cells in each
#'  iteration are filled with different values. In step 3 EM-AMMI uses those
#'  effects to re-estimate cells marked as missed (as default,
#'  simplified.model=FALSE). It is, however, possible that this procedure will
#'  not converge. Thus the user is offered a simplified EM-AMMI procedure that
#'  calculates the general mean and effects of rows and columns only in the
#'  first iteration and in next iterations uses these values
#'  (simplified.model=TRUE). In this simplified procedure the initial values
#'  affect the outcome (whilst EM-AMMI results usually do not depend on initial
#'  values). For the simplified procedure the number of iterations to
#'  convergence is usually smaller and, furthermore, convergence will be reached
#'  even in some cases where the regular procedure fails. If the regular
#'  procedure does not converge for the standard initial values (see the
#'  description of the argument initial.values), the simplified model can be
#'  used to determine a better set of initial values.
#'@references Paderewski, J. (2013). \emph{An R function for imputation of
#'  missing cells in two-way initial.values), the simplified model can be used
#'  to determine a better set of initial values. data sets by EM-AMMI
#'  algorithm.}. Communications in Biometry and Crop Science 8, 60–69.
#'@return A list containing:
#'  \itemize{
#'    \item  X: the imputed matrix (filled in with the missing values estimated
#'    by the EM-AMMI procedure);
#'    \item PC.SS: the sum of squares representing variation explained
#'    by the principal components (the squares of eigenvalues of singular value
#'    decomposition);
#'    \item iteration: the final number of iterations;
#'    \item precision.final: the maximum change of the estimated values for
#'    missing cells in the last step of iteration (the precision of convergence)
#'    . If the algorithm converged, this value is slightly smaller than the
#'    argument precision;
#'    \item PC.nb.final: a number of principal components that were eventually
#'    used by the EM.AMMI() function. The function checks if there are too many
#'    missing cells to unambiguously compute the parameters by the SVD
#'    decomposition (Gauch and Zobel, 1990). In that case the final number of
#'    principal components used is smaller than that which was passed on to the
#'    function through the argument PC.nb;
#'    \item convergence: the value TRUE means that the algorithm converged in
#'    the last iteration.
#'  }
#'@keywords internal
#'

EM.AMMI<-function(X, PC.nb=1, initial.values=NA, precision=0.01,
                  max.iter=1000, change.factor=1, simplified.model=FALSE)
{
  if (missing(X)) stop("Need to provide X data frame or matrix")
  stopifnot(
    class(X) %in% c("matrix", "data.frame"),
    class(PC.nb)=="numeric",
    class(initial.values) == "matrix" | is.na(initial.values),
    class(initial.values) %in% c("matrix", "logical"),
    class(precision)=="numeric",
    class(max.iter)=="numeric",
    class(change.factor)=="numeric",
    class(simplified.model) == "logical"
  )

  X<-as.matrix(X)
  X.missing<-matrix(1,nrow(X),ncol(X))
  X.missing[is.na(X)]<-0
  max.IPC=min(c(rowSums(X.missing),colSums(X.missing)))-1
  if (max.IPC<PC.nb)
  {PC.nb.used<-max.IPC} else {PC.nb.used<-PC.nb}
  X.ini<-X
  if (length(initial.values)==1)
  {
    if (!is.na(initial.values))
    {
      initial.values<-matrix(initial.values,nrow(X),ncol(X))
      X.ini[is.na(X.ini)]<-initial.values[is.na(X)]
    } else {
      X.mean<-mean(c(X),na.rm = TRUE)
      row.m<-matrix(rowMeans(X,na.rm = TRUE),nrow(X),ncol(X))
      col.m<-t(matrix(colMeans(X,na.rm = TRUE),ncol(X),nrow(X)))
      estimated<-(-X.mean)+row.m+col.m
      X.ini[is.na(X.ini)]<-estimated[is.na(X)]
    }
  } else {
    X.ini[is.na(X.ini)]<-initial.values[is.na(X)]
  }
  iteration<-1
  X.new<-X.ini
  change<-precision+1
  while ((change>precision)&(iteration<max.iter))
  {
    if (iteration==1 | !simplified.model)
    {
      x.mean<-mean(X.new)
      X.new.Ie<-X.new-x.mean
      X.new.Ie<-scale(X.new.Ie,center = TRUE, scale = FALSE)
      x.col.center<-attr(X.new.Ie,"scaled:center")
      X.new.Ie<-t(scale(t(X.new.Ie),center = TRUE, scale = FALSE))
      x.row.center<-attr(X.new.Ie,"scaled:center")
    } else { X.new.Ie<-X.new-x.mean-row.eff-col.eff }
    if (PC.nb.used>=1)
    {
      SVD <- La.svd(X.new.Ie)
      SVD$d<-(SVD$d[1:PC.nb.used])
      SVD$u<-SVD$u[,1:PC.nb.used]
      SVD$v<-SVD$v[1:PC.nb.used,]
      diag.l<-diag(SVD$d,nrow=PC.nb.used)
      interaction.adj<-SVD$u%*%diag.l%*%SVD$v
    } else interaction.adj<-0
    if (iteration==1 | !simplified.model)
    {
      row.eff<-matrix(x.row.center,nrow(X),ncol(X))
      col.eff<-t(matrix(x.col.center,ncol(X),nrow(X)))
    }
    X.next<-X.new
    X.next[is.na(X)]<-(x.mean+row.eff+col.eff+interaction.adj)[is.na(X)]
    change<-max(abs(c( (X.next-X.new)[is.na(X)] )))
    iteration<-iteration+1
    X.new<-change.factor*X.next+(1-change.factor)*X.new
  }
  if (change<=precision) {state=TRUE} else {state=FALSE}
  if (PC.nb.used<PC.nb) {state=FALSE}
  if (PC.nb.used==0) {SVD<-list(d=0)}
  return(list(X=X.new,iteration=iteration,precision.final=change,PC.nb.final=PC.nb.used, convergence=state))
}

#' Imputation of missing cells in two-way data sets
#'
#' Missing values are not allowed by the AMMI, GGE or SREG methods. This function
#' provides several methods to impute missing observations in data from
#' multi-environment trials and to subsequently adjust the mentioned methods.
#'
#' @param Data dataframe containing genotypes, environments, repetitions (if any)
#'   and the phenotypic trait of interest. Other variables that will not be used
#'   in the analysis can be present.
#' @param genotype column name containing genotypes.
#' @param environment column name containing environments.
#' @param response column name containing the phenotypic trait.
#' @param rep column name containing replications. If this argument is NULL,
#'   there are no replications available in the data. Defaults to NULL.
#' @param type imputation method. Either "EM-AMMI", "EM-GGE", "EM-SREG", "EM-bSREG",
#'   "Gabriel", "Eigenvector", "WGabriel", "EM-PCA". Defaults to "EM-AMMI".
#' @param nPC number of components used to predict the missing values.
#'   Default to 2.
#' @param initial.values initial values of the missing cells. It can be a single
#'   value or a vector of length equal to the number of missing cells.
#' @param precision threshold for assessing convergence.
#' @param maxiter maximum number of iteration for the algorithm.
#' @param change.factor When `change.factor` is equal to 1, the previous
#'   approximation is changed with the new values (standard EM). Smaller values
#'   can help convergence if changes are cyclic.
#' @param simplified.model logical. If TRUE, calculates effects only in the first 
#'   iteration to speed up convergence or help in cases where the regular 
#'   procedure fails.
#' @param scale boolean. By default TRUE for "EM-PCA".
#' @param method "Regularized" or "EM" for "EM-PCA".
#' @param row.w row weights for "EM-PCA".
#' @param coeff.ridge ridge coefficient for "EM-PCA".
#' @param seed integer for random initialization in "EM-PCA".
#' @param nb.init number of random initializations for "EM-PCA".
#' @param Winf lower weight for WGabriel.
#' @param Wsup upper weight for WGabriel.
#'
#'@return A matrix of the imputed data.
#'
#'@details
#'
#'Often, multi-environment experiments are unbalanced because several genotypes
#'are not tested in some environments. Several methodologies have been proposed
#'in order to solve this lack of balance caused by missing values, some of which
#'are included in this function:
#'
#'\itemize{
#'\item EM-AMMI: an iterative scheme built round the above procedure is used to
#'obtain AMMI imputations from the EM algorithm. The additive parameters are
#'initially set by computing the grand mean, genotype means and environment
#'means obtained from the observed data. The residuals for the observed cells
#'are initialized as the cell mean minus the genotype mean minus the environment
#'mean plus the grand mean, and interactions for the missing positions are
#'initially set to zero. The initial multiplicative parameters are obtained from
#'the SVD of this matrix of residuals, and the missing values are filled by the
#'appropriate AMMI estimates. In subsequent iterations, the usual AMMI procedure
#'is applied to the completed matrix and the missing values are updated by the
#'corresponding AMMI estimates. The arguments used for this method
#'are:initial.values, precision, maxiter, change.factor and simplified.model
#'
#'\item EM-GGE: Iterative SVD-based imputation focusing on G+GE.
#' 
#' \item EM-SREG: Iterative algorithm using the Sites Regression model. Supports variants 
#'  like standard SVD and Bayesian PCA (EM-bSREG). 
#' 
#'\item Gabriel: combines regression and lower-rank approximation using SVD.
#'This method initially replaces the missing cells by arbitrary values, and
#'subsequently the imputations are refined through an iterative scheme that
#'defines a partition of the matrix for each missing value in turn and uses a
#'linear regression of columns (or rows) to obtain the new imputation. The
#'arguments used for this method is only the dataframe.
#'
#'\item WGabriel: is a a modification of Gabriel method that uses weights chosen
#'by cross-validation. The arguments used for this method are Winf and Wsup.
#'
#'\item EM-PCA: impute the missing entries of a mixed data using the iterative
#'PCA algorithm. The algorithm first consists imputing missing values with
#'initial values. The second step of the iterative PCA algorithm is to perform
#'PCA on the completed dataset to estimate the parameters. Then, it imputes the
#'missing values with the reconstruction formulae of order nPC (the fitted
#'matrix computed with nPC components for the scores and loadings). These steps
#'of estimation of the parameters via PCA and imputation of the missing values
#'using the fitted matrix are iterate until convergence. The arguments used for
#'this methods are: nPC, scale, method, row.w, coeff.ridge, precision, seed,
#'nb.init and maxiter
#'
#'}
#'
#'
#'@references Paderewski, J. (2013). \emph{An R function for imputation of missing
#'  cells in two-way data sets by EM-AMMI algorithm}. Communications in Biometry
#'  and Crop Science 8, 60–69.
#'  
#' @references 
#' Yan, W. (2013). \emph{Biplot analysis of incomplete two-way data}. 
#' Crop Science, 53(1), 48-57. \doi{10.2135/cropsci2012.05.0301}
#' 
#' Arciniegas-Alarcón, S., García-Peña, M., Krzanowski, W., & Dias, C. T. S. 
#' (2014b). \emph{An alternative methodology for imputing missing data in 
#' trials with genotype-by-environment interaction: some new aspects}. 
#' Biometrical Letters, 51(2), 75-88. \doi{10.2478/bile-2014-0006}
#' 
#' @references 
#' Angelini, J., Cervigni, G. D. L., & Quaglino, M. B. (2024). \emph{New 
#' imputation methodologies for genotype-by-environment data: an extensive 
#' study of properties of estimators}. Euphytica, 220(6), 92. 
#' \doi{10.1007/s10681-024-03344-z}
#' 
#'@references Julie Josse, Francois Husson (2016). missMDA: A Package for
#'  Handling Missing Values in Multivariate Data Analysis. Journal of
#'  Statistical Software 70, 1-31.
#'@references Arciniegas-Alarcón S., García-Peña M., Dias C.T.S., Krzanowski
#'  W.J. (2010). \emph{An alternative methodology for imputing missing data in
#'  trials with genotype-by-environment interaction}. Biometrical Letters 47,
#'  1–14.
#'@references Arciniegas-Alarcón S., García-Peña M., Krzanowski W.J., Dias
#'  C.T.S. (2014). \emph{An alternative methodology for imputing missing data in
#'  trials with genotype-by-environment interaction: some new aspects.}
#'  Biometrical Letters 51, 75-88.
#'
#'@export
#'
#' @examples
#' library(geneticae)
#' # Data without replications
#' library(agridat)
#' data(yan.winterwheat)
#'
#' # generating missing values
#' yan.winterwheat[1,3]<-NA
#' yan.winterwheat[3,3]<-NA
#' yan.winterwheat[2,3]<-NA
#'
#' imputation(yan.winterwheat, genotype = "gen", environment = "env",
#'            response = "yield", type = "EM-AMMI")
#'
#' # Data with replications
#' data(plrv)
#' head(plrv)
#' plrv$Yield[plrv$Locality == "Ayac" & plrv$Rep %in% c(1, 2, 3) & plrv$Genotype == '102.18'] <- NA
#' 
#' imputation(plrv, nPC = 2,genotype = "Genotype", environment = "Locality", 
#'            response = "Yield", rep ='Rep', type = "EM-AMMI")
#'            
#' imputation(plrv, genotype = "Genotype", environment = "Locality", 
#'            response = "Yield", rep ='Rep', type = "EM-SREG")
#'
#'@importFrom stats var
#'@importFrom missMDA imputePCA
#'@importFrom dplyr group_by summarise rename %>% pull select all_of
#'@importFrom rlang sym
#'@importFrom tidyr pivot_wider
#'
imputation <- function(Data, genotype="gen", environment="env", response="yield", rep=NULL, 
                       type="EM-AMMI", nPC=2, initial.values=NA, precision=0.01, 
                       maxiter=1000, change.factor=1, simplified.model=FALSE,
                       scale = TRUE, method = "EM", row.w = NULL, coeff.ridge = 1, 
                       seed = NULL, nb.init = 1, Winf=0.8, Wsup=1) {
  
  if (missing(Data)) stop("Need to provide Data data frame")
  if (!any(is.na(Data))) stop("There are no missing data in input data frame")
  
  stopifnot(
    is.data.frame(Data),
    type %in% c("EM-AMMI", "EM-GGE", "EM-SREG", "EM-bSREG", "Gabriel", "Eigenvector", "WGabriel", "EM-PCA")
  )

  if(!is.null(rep)){
    Data_wide <- Data %>%
      dplyr::group_by(!!rlang::sym(genotype), !!rlang::sym(environment)) %>%
      dplyr::summarise(mean_resp = mean(!!rlang::sym(response), na.rm = TRUE), .groups = "drop") %>%
      tidyr::pivot_wider(names_from = !!rlang::sym(environment), values_from = mean_resp) %>%
      as.data.frame()
  } else {
    Data_wide <- Data %>%
      tidyr::pivot_wider(names_from = !!rlang::sym(environment), values_from = !!rlang::sym(response)) %>%
      as.data.frame()
  }

  rownames(Data_wide) <- Data_wide[[genotype]]

  Data_mat <- Data_wide %>% dplyr::select(-dplyr::all_of(genotype))

  Data_mat <- as.matrix(Data_mat)
  Data_mat[is.na(Data_mat)] <- NA

  Data_mat <- as.data.frame(Data_mat)
  
  
  if(type == "EM-AMMI"){
    matrix_res <- EM.AMMI(Data_mat, PC.nb = nPC, initial.values = initial.values, 
                          precision = precision, max.iter = maxiter, 
                          change.factor = change.factor, 
                          simplified.model = simplified.model)$X
  }
  
  if(type == "EM-GGE"){
    matrix_res <- BiplotImputfun(Data_mat, precision = precision, 
                                 max.iter = maxiter, n_pc = nPC)$Biplot.imput
  }
  
  if(type %in% c("EM-SREG", "EM-bSREG")){
    matrix_res <- EM.SREG(Data_mat, PC.nb = nPC, initial.values = initial.values,
                          precision = precision, max.iter = maxiter, 
                          change.factor = change.factor, 
                          simplified.model = simplified.model, 
                          type = type)$X
  }
  
  if(type == "Gabriel"){
    matrix_res <- Gabriel.Calinski(Data_mat)$GabrielImput
  }
  
  else if(type == "Eigenvector"){
    matrix_res <- Eigenvectorfun(Data_mat, f = nPC)$EigenImputaciones
  }
  
  if(type == "WGabriel"){
    matrix_res <- WGabriel(Data_mat, Winf, Wsup)$GabrielWImput
  }
  
  if(type == "EM-PCA"){
    matrix_res <- missMDA::imputePCA(Data_mat, ncp = nPC, scale = scale, method = method,
                                     row.w = row.w, coeff.ridge = coeff.ridge, 
                                     threshold = precision, seed = seed, 
                                     nb.init = nb.init, maxiter = maxiter)$completeObs
  }
  
  return(as.matrix(matrix_res))
}

library(geneticae)
library(agridat)
library(dplyr)

# 1. Preparación de datos
data(yan.winterwheat)
data(plrv)

# Dataset con NAs
dat_na <- yan.winterwheat
dat_na[1, 3] <- NA
dat_na[5, 3] <- NA

test_that("Validación de errores de entrada en imputación", {
  expect_error(imputation(yan.winterwheat), "There are no missing data in input data frame")
  expect_error(imputation(genotype="gen"), "Need to provide Data data frame")
  expect_error(imputation(dat_na, type = "METODO_FALSO"))
})

test_that("Consistencia de los métodos de imputación principales", {
  res_ammi <- imputation(dat_na, genotype="gen", environment="env", response="yield")
  expect_true(is.matrix(res_ammi))
  expect_false(any(is.na(res_ammi)))
  expect_equal(nrow(res_ammi), length(unique(yan.winterwheat$gen)))
})

test_that("Prueba de métodos específicos (EM-PCA, Gabriel, GGE)", {
  # EM-PCA
  expect_false(any(is.na(imputation(dat_na, type = "EM-PCA", nPC = 1))))
  
  # Gabriel
  expect_false(any(is.na(imputation(dat_na, type = "Gabriel"))))
  
  # EM-GGE
  expect_false(any(is.na(imputation(dat_na, type = "EM-GGE"))))
})

test_that("Manejo de repeticiones y balanceo", {
  # Usamos suppressWarnings para el aviso interno de max() -Inf en la convergencia
  res_rep <- suppressWarnings(
    imputation(plrv %>% mutate(Yield = ifelse(row_number() == 2, NA, Yield)), 
               genotype = "Genotype", environment = "Locality", 
               response = "Yield", rep = "Rep", type = "EM-AMMI")
  )
  
  expect_true(is.matrix(res_rep))
  expect_equal(nrow(res_rep), length(unique(plrv$Genotype)))
  expect_false(any(is.na(res_rep)))
})

test_that("Argumentos técnicos e integridad", {
  expect_silent(suppressWarnings(imputation(dat_na, nPC = 1, precision = 0.1)))
  
  # Igualdad por defecto
  expect_equal(imputation(dat_na), imputation(dat_na, type = "EM-AMMI"))
})

#' EM-SREG Imputation Method
#'
#' @description Iterative algorithm to impute missing values in two-way tables 
#'   using the Sites Regression (SREG) model. It supports several variants 
#'   including standard SVD and Bayesian PCA.
#'
#' @param X A data frame or matrix with genotypes in rows and environments in columns.
#' @param PC.nb Number of principal components to be used. Default is 1.
#' @param initial.values (optional) Initial values for missing cells. If NA, 
#'   initial values are obtained from column means (environment effects).
#' @param precision Convergence threshold. Default is 0.01.
#' @param max.iter Maximum number of iterations. Default is 1000.
#' @param change.factor Step size for updating missing values (standard is 1).
#' @param simplified.model Logical. If TRUE, effects are only calculated in the first iteration.
#' @param type Method type: "EM-SREG" (Standard), "EM-bSREG" (Bayesian).
#'
#' @references 
#' Angelini, J., Cervigni, G. D. L., & Quaglino, M. B. (2024). \emph{New 
#' imputation methodologies for genotype-by-environment data: an extensive 
#' study of properties of estimators}. Euphytica, 220(6), 92. 
#' \doi{10.1007/s10681-024-03344-z}
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{X}: The imputed matrix.
#'   \item \code{iter}: The number of iterations until convergence.
#' }
#' 
#' @importFrom stats sd
#' @importFrom pcaMethods pca scores loadings
#' @keywords internal
#' @export
EM.SREG <- function(X, PC.nb = 1, initial.values = NA, precision = 0.01,
                    max.iter = 1000, change.factor = 1, simplified.model = FALSE, 
                    type = c("EM-SREG", "EM-bSREG")) {
  
  type <- match.arg(type)
  X <- as.matrix(X)
  indicamissing <- is.na(X)
  
  # Determinar número máximo de componentes posibles
  X_missing_mask <- !indicamissing
  max.IPC <- min(rowSums(X_missing_mask), colSums(X_missing_mask) - 1)
  PC.nb.used <- if (max.IPC < PC.nb) max.IPC else PC.nb
  
  # --- 1. Inicialización ---
  X.new <- X
  if (length(initial.values) == 1) {
    if (!is.na(initial.values)) {
      X.new[indicamissing] <- initial.values
    } else {
      # Efecto de columna (Ambiente) como inicialización para SREG
      col_m <- colMeans(X, na.rm = TRUE)
      X.new[indicamissing] <- col_m[col(X)[indicamissing]]
    }
  } else {
    X.new[indicamissing] <- initial.values[indicamissing]
  }
  
  iteration <- 1
  change <- precision + 1
  
  # --- 2. Algoritmo Iterativo ---
  while ((change > precision) && (iteration < max.iter)) {
    
    if (iteration == 1 || !simplified.model) {
      x.mean <- mean(X.new)
      # Centrado por columna (SREG se centra en el ambiente)
      X_centered <- scale(X.new - x.mean, center = TRUE, scale = FALSE)
      col_eff <- attr(X_centered, "scaled:center")
    } else {
      X_centered <- X.new - x.mean - matrix(col_eff, nrow(X.new), ncol(X.new), byrow = TRUE)
    }
    
    # --- Ajuste según tipo de SREG ---
    if (PC.nb.used >= 1) {
      if (type == "EM-SREG") {
        # SVD estándar
        SVD <- svd(X_centered)
        interaction.adj <- SVD$u[, 1:PC.nb.used, drop = FALSE] %*% 
          diag(SVD$d[1:PC.nb.used], nrow = PC.nb.used) %*% 
          t(SVD$v[, 1:PC.nb.used, drop = FALSE])
      } else if (type == "EM-bSREG") {
        # Requiere paquete pcaMethods
        if (!requireNamespace("pcaMethods", quietly = TRUE)) stop("Package 'pcaMethods' needed for EM-bSREG.")
        Xpca <- pcaMethods::pca(X_centered, method = "bpca", nPcs = PC.nb.used, center = FALSE)
        interaction.adj <- pcaMethods::scores(Xpca) %*% t(pcaMethods::loadings(Xpca))
      } else {
        # Placeholder para los otros tipos (gSREG, ppSREG, etc.)
        interaction.adj <- 0 
      }
    } else {
      interaction.adj <- 0
    }
    
    # Reconstrucción del modelo SREG: Media + Efecto Ambiente + Interacción
    X.fitted <- x.mean + matrix(col_eff, nrow(X), ncol(X), byrow = TRUE) + interaction.adj
    
    # Calcular cambio solo en los faltantes
    X.next <- X.new
    X.next[indicamissing] <- X.fitted[indicamissing]
    
    change <- max(abs(X.next[indicamissing] - X.new[indicamissing]))
    
    # Aplicar factor de cambio y actualizar
    X.new <- change.factor * X.next + (1 - change.factor) * X.new
    iteration <- iteration + 1
  }
  
  return(list(X = X.new, iter = iteration))
}

#' Robust AMMI Model Fitting
#'
#' @description Fits a classical or robust Additive Main effects and
#'   Multiplicative Interaction (AMMI) model for genotype-by-environment data.
#'
#' @param Data a dataframe with genotypes, environments, repetitions (if any)
#'   and the phenotypic trait of interest. Other variables that will not be used
#'   in the analysis can be included.
#' @param genotype column name containing genotypes. Defaults to `"gen"`.
#' @param environment column name containing environments. Defaults to `"env"`.
#' @param response column name containing the phenotypic trait of interest.
#'   Defaults to `"Y"`.
#' @param rep column name containing replications. If this argument is `NULL`
#'   (default), it is assumed that the data already contains means per genotype
#'   in each environment. If provided, means are calculated automatically.
#' @param Ncomp number of principal components to retain for the interaction
#'   part. Defaults to 2.
#' @param type method for fitting the AMMI model: `"AMMI"` (classical),
#'   `"rAMMI"`, `"hAMMI"`, `"gAMMI"`, `"lAMMI"` or `"ppAMMI"` (robust variants).
#'   Defaults to `"AMMI"`.
#'
#' @details To overcome the problem of data contamination with outlying
#' observations, Rodrigues, Monteiro and Lourenco (2015) propose a robust AMMI
#' model based on the M-Huber estimator and robust SVD/PCA procedures.
#'
#' The `type` argument allows choosing between several robust strategies:
#' \itemize{
#'   \item \bold{AMMI}: Classical AMMI model using Least Squares and standard SVD.
#'   \item \bold{rAMMI}: Uses the L1 norm instead of the L2 norm to compute a
#'     robust approximation to the SVD (via \pkg{pcaMethods}).
#'   \item \bold{hAMMI}: Uses the Hubert's approach (PcaHubert) combining
#'     projection-pursuit and robust covariance estimation.
#'   \item \bold{gAMMI}: Uses the Grid search algorithm for PCA (PcaGrid).
#'   \item \bold{lAMMI}: Performs PCA on the data projected onto a unit sphere (PcaLocantore).
#'   \item \bold{ppAMMI}: Uses projection-pursuit (PcaProj) to calculate robust
#'     eigenvalues and eigenvectors.
#' }
#'
#' @return A list of class \code{rAMMI} containing: \item{gen_scores}{Matrix of
#'   genotype scores (U * D).} \item{env_scores}{Matrix of environment loadings
#'   (V).} \item{eigenvalues}{Vector of singular values for the retained
#'   components.} \item{gen_labels}{Names of the genotypes.}
#'   \item{env_labels}{Names of the environments.} \item{Ncomp}{Number of
#'   principal components used.} \item{type}{The fitting method used.}
#'   \item{vartotal}{Total variance explained by the multiplicative terms.}
#'
#' @references Rodrigues P.C., Monteiro A., Lourenco V.M. (2015). A robust AMMI
#'    model for the analysis of genotype-by-environment data. Bioinformatics 32,
#'    58-66.
#'
#' @importFrom MASS rlm
#' @importFrom pcaMethods robustSvd
#' @importFrom rrcov PcaHubert PcaGrid PcaLocantore PcaProj
#' @importFrom stats lm residuals as.formula
#' @importFrom dplyr group_by summarise pull arrange %>%
#' @importFrom rlang sym
#' @export
#'
#' @examples
#' library(agridat)
#' data(yan.winterwheat)
#'
#' # Classical AMMI
#' mod_ammi <- rAMMIModel(yan.winterwheat, genotype = "gen",
#'                        environment = "env", response = "yield", type = "AMMI")
#'
#' # Robust AMMI (using Hubert's method)
#' mod_rammi <- rAMMIModel(yan.winterwheat, genotype = "gen",
#'                         environment = "env", response = "yield", type = "hAMMI")
#'
#' 
rAMMIModel <- function(Data, genotype = "gen", environment = "env", response = "Y", rep = NULL, Ncomp = 2, type = "AMMI") {
  
  if (missing(Data)) stop("Need to provide Data data frame")
  if (any(is.na(Data))) stop("Missing data in input data frame")
  

  Data_avg <- Data %>%
    group_by(!!sym(genotype), !!sym(environment)) %>%
    summarise(y = mean(!!sym(response), na.rm = TRUE), .groups = "drop") %>%
    arrange(!!sym(environment), !!sym(genotype))
  
  gen <- as.factor(Data_avg[[genotype]])
  env <- as.factor(Data_avg[[environment]])
  y   <- Data_avg$y
  
  Ngen <- nlevels(gen)
  Nenv <- nlevels(env)

  
  if (type == "AMMI") {
    lm.x <- lm(y ~ gen + env, data = Data_avg)
    residuals.x <- matrix(residuals(lm.x), nrow = Ngen, ncol = Nenv)
  }

  if (type != "AMMI") {
    rlm.x <- MASS::rlm(y ~ gen + env, data = Data_avg, maxit = 100) 
    rresiduals.x <- matrix(residuals(rlm.x), nrow = Ngen, ncol = Nenv)
  }
  
  if (type == "AMMI") {
    svd.x <- svd(residuals.x)
    singlevalue <- svd.x$d
    loading <- svd.x$v
    score <- svd.x$u %*% diag(svd.x$d)
  } else if (type == "rAMMI") {
    svd_result <- tryCatch({
      robustSvd(rresiduals.x)
    }, error = function(e) {
      stop(paste("The robustSvd algorithm did not converge with this data.",
                 "Try using type='AMMI' or rrcov methods like 'hAMMI'."))
    })
    singlevalue <- svd_result$d
    loading <- svd_result$v
    score <- svd_result$u %*% diag(svd_result$d)
  } else {
    res_pca <- switch(type,
                      "hAMMI"  = PcaHubert(rresiduals.x, mcd = FALSE),
                      "gAMMI"  = PcaGrid(rresiduals.x),
                      "lAMMI"  = PcaLocantore(rresiduals.x),
                      "ppAMMI" = PcaProj(rresiduals.x))
    
    singlevalue <- if(type == "lAMMI") res_pca@eigenvalues else sqrt(res_pca@eigenvalues)
    loading     <- res_pca@loadings
    score       <- res_pca@scores
    singlevalue <- singlevalue * sqrt(nrow(score))
  }
  
  res <- list(
    gen_scores = score,
    env_scores = loading,
    eigenvalues = singlevalue,
    gen_labels = levels(gen),
    env_labels = levels(env),
    Ncomp = Ncomp,
    type = type,
    vartotal = sum(singlevalue^2)
  )
  class(res) <- "rAMMI"
  return(res)
}

library(testthat)
library(dplyr)
library(geneticae)

# Función auxiliar para generar datos limpios
get_simple_test_data <- function() {
  set.seed(123)
  expand.grid(
    Genotype = paste0("G", 1:8), 
    Locality = paste0("L", 1:4), 
    Rep = 1:2
  ) %>%
    mutate(Yield = 10 + rnorm(n()))
}

test_that("rAMMIModel procesa correctamente los datos y promedios", {
  df <- get_simple_test_data()
  
  # Probamos el modelo base (AMMI clásico)
  # Esto valida que dplyr, group_by, summarise y arrange funcionen bien
  res <- rAMMIModel(df, "Genotype", "Locality", "Yield", rep = "Rep", type = "AMMI")
  
  expect_s3_class(res, "rAMMI")
  expect_equal(nrow(res$gen_scores), 8)
  expect_equal(nrow(res$env_scores), 4)
  expect_equal(res$type, "AMMI")
})

test_that("rAMMIModel funciona con métodos estables de rrcov", {
  df <- get_simple_test_data()
  
  # Testeamos hAMMI y gAMMI que suelen ser más estables que robustSvd
  estables <- c("hAMMI", "gAMMI")
  
  for (t in estables) {
    expect_error(
      rAMMIModel(df, "Genotype", "Locality", "Yield", type = t),
      NA,
      info = paste("Falló el método estable:", t)
    )
  }
})

test_that("rAMMIModel lanza error con NAs", {
  df <- get_simple_test_data()
  df$Yield[1] <- NA
  
  expect_error(
    rAMMIModel(df, "Genotype", "Locality", "Yield"),
    "Missing data in input data frame"
  )
})

library(testthat)
library(ggplot2)
library(geneticae)

# --- FUNCIÓN AUXILIAR ESTABLE ---
generar_modelo_test <- function() {
  # Datos con estructura aditiva clara para facilitar convergencia
  set.seed(123)
  df <- expand.grid(Genotype = paste0("G", 1:6), 
                    Locality = paste0("L", 1:4)) %>%
    mutate(Yield = 10 + 
             as.numeric(as.factor(Genotype)) * 0.5 + 
             as.numeric(as.factor(Locality)) * 1.5 + 
             rnorm(n(), sd = 0.1))
  
  # Usamos AMMI para los tests de interfaz (es el más rápido y estable)
  return(rAMMIModel(df, "Genotype", "Locality", "Yield", type = "AMMI"))
}

# --- TESTS ---

test_that("rAMMIPlot genera un objeto ggplot válido", {
  modelo <- generar_modelo_test()
  p <- rAMMIPlot(modelo)
  expect_s3_class(p, "ggplot")
})

test_that("rAMMIPlot respeta los argumentos de personalización", {
  modelo <- generar_modelo_test()
  p_custom <- rAMMIPlot(modelo, titles = FALSE, axislabels = FALSE, footnote = FALSE)
  
  expect_null(p_custom$labels$title)
  expect_null(p_custom$labels$caption)
  
  # Verificamos que no haya porcentajes si axislabels = FALSE
  label_x <- p_custom$labels$x
  if (!is.null(label_x)) expect_false(grepl("%", label_x))
})

test_that("rAMMIPlot muestra porcentajes cuando axislabels es TRUE", {
  modelo <- generar_modelo_test()
  p <- rAMMIPlot(modelo, axislabels = TRUE)
  
  expect_true(grepl("%", p$labels$x))
  expect_true(grepl("%", p$labels$y))
})

test_that("rAMMIPlot contiene las capas esenciales", {
  modelo <- generar_modelo_test()
  p <- rAMMIPlot(modelo)
  
  capas <- sapply(p$layers, function(x) class(x$geom)[1])
  expect_true(any(capas == "GeomSegment"))
  expect_true(any(capas == "GeomText"))
})

#' Biplot Imputation Method
#'
#' @description This function implements the Biplot imputation method as 
#' proposed by Yan (2013). It is an iterative algorithm that uses the singular 
#' value decomposition (SVD) to impute missing values in a genotype by 
#' environment matrix.
#'
#' @param X A data frame or matrix with genotypes in rows and 
#'   environments in columns.
#' @param precision (optional) Convergence threshold. The algorithm stops 
#'   when the relative change in imputed values is less than this value. 
#'   Default is 0.01.
#' @param max.iter (optional) Maximum number of iterations. Default is 1000.
#' @param n_pc Number of principal components to use for imputation. 
#'   Default is 2.
#'
#' @references 
#' Yan, W. (2013). \emph{Biplot analysis of incomplete two-way data}. 
#' Crop Science, 53(1), 48-57. \doi{10.2135/cropsci2012.05.0301}
#' 
#' Arciniegas-Alarcón, S., García-Peña, M., Krzanowski, W., & Dias, C. T. S. 
#' (2014b). \emph{An alternative methodology for imputing missing data in 
#' trials with genotype-by-environment interaction: some new aspects}. 
#' Biometrical Letters, 51(2), 75-88. \doi{10.2478/bile-2014-0006}
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{X_imputed}: The final matrix with missing values filled.
#'   \item \code{iteration}: Total number of iterations performed.
#'   \item \code{convergence}: Final relative change reached.
#'   \item \code{fitted}: The fitted values from the AMMI/Biplot model.
#' }
#' 
#' @importFrom stats sd
#' @keywords internal
#' @export


BiplotImputfun <- function(X, precision = 0.01, max.iter = 1000, n_pc = 2) {
  
  X_mat <- as.matrix(X)
  indicamissing <- is.na(X_mat)
  indicaobservEM <- !indicamissing
  
  col_means <- colMeans(X_mat, na.rm = TRUE)
  X_original <- X_mat
  X_original[indicamissing] <- col_means[col(X_mat)[indicamissing]]
  
  iterconverg <- 1
  stabilitycrit <- 1 + precision
  
  RSS_N1 <- sqrt(sum(X_original[indicaobservEM]^2) / sum(indicaobservEM))
  
  while (stabilitycrit > precision && iterconverg <= max.iter) {
    
    C_original <- matrix(colMeans(X_original), nrow = nrow(X_original), ncol = ncol(X_original), byrow = TRUE)
    
    desvest_original <- matrix(apply(X_original, 2, stats::sd), 
                               nrow = nrow(X_original), ncol = ncol(X_original), byrow = TRUE)
    
    pij_original <- (X_original - C_original) / desvest_original
    
    dvs_pij <- svd(pij_original)
    
    U <- dvs_pij$u[, 1:n_pc, drop = FALSE]
    D <- diag(dvs_pij$d[1:n_pc], nrow = n_pc)
    V <- dvs_pij$v[, 1:n_pc, drop = FALSE]
    
    YanImputation <- U %*% D %*% t(V)
    
    Y_EMammi1 <- C_original + (YanImputation * desvest_original)
    
    RSS_N <- sqrt(sum((X_original[indicamissing] - Y_EMammi1[indicamissing])^2) / sum(indicamissing))
    stabilitycrit <- RSS_N / RSS_N1
    
    X_original[indicamissing] <- Y_EMammi1[indicamissing]
    
    iterconverg <- iterconverg + 1
  }
  
  return(list(
    Biplot.iter = iterconverg - 1,
    Biplot.converg = stabilitycrit,
    Biplot.imput = X_original,
    X2.Chapeu = Y_EMammi1
  ))
}

#' AMMI Biplots with \pkg{ggplot2}
#'
#' @description Generates a high-quality biplot (PC1 vs PC2) for classical or 
#'   robust AMMI models using the \pkg{ggplot2} framework.
#'
#' @param model_res an object of class \code{rAMMI} generated by \code{\link{rAMMIModel}}.
#' @param colGen color for genotype labels. Defaults to `"gray47"`.
#' @param colEnv color for environment labels and vectors. Defaults to `"darkred"`.
#' @param sizeGen text size for genotype labels. Defaults to 6.
#' @param sizeEnv text size for environment labels. Defaults to 6.
#' @param titles logical. If `TRUE` (default), a title indicating the model 
#'   type is added to the plot.
#' @param footnote logical. If `TRUE` (default), a caption with the percentage 
#'   of explained GxE variation is added at the bottom.
#' @param axis_expand numeric value to expand the axis limits. Useful to prevent 
#'   labels from being cut off. Defaults to 1.2.
#' @param limits logical. If `TRUE` (default), uses \code{coord_fixed()} to 
#'   maintain a 1:1 aspect ratio between axes.
#' @param axes logical. If `TRUE` (default), draws dashed horizontal and 
#'   vertical lines passing through the origin (0,0).
#' @param axislabels logical. If `TRUE` (default), includes axis titles with 
#'   the percentage of variance explained by each principal component.
#'
#' @details 
#' the biplot is constructed using a scaling factor of 0.5 (symmetric scaling), 
#' which allows representing both genotypes and environments in the same 
#' algebraic space. Genotypes are displayed as points (text), and environments 
#' are represented as vectors from the origin.
#'
#' @return A \pkg{ggplot2} object. This allows further customization using 
#'   standard ggplot layers (e.g., \code{+ theme_bw()}).
#'
#' @import ggplot2
#' @export
#' 
#' @examples
#' library(agridat)
#' data(yan.winterwheat)
#' 
#' # Classical AMMI
#' mod_ammi <- rAMMIModel(yan.winterwheat, genotype = "gen", 
#'                        environment = "env", response = "yield", type = "AMMI")
#' 
#' rAMMIPlot(mod_ammi,sizeGen=4,sizeEnv=4)
#' 
#' data(plrv)
#' mod_ammi_rep <- rAMMIModel(plrv, genotype="Genotype", environment="Locality", 
#'                         response="Yield", rep="Rep", type="AMMI")
#' rAMMIPlot(mod_ammi_rep,sizeGen=4,sizeEnv=4)
#'
rAMMIPlot <- function(model_res, colGen = "gray47", colEnv = "darkred", sizeGen = 6, sizeEnv = 6, 
                      titles = TRUE, footnote = TRUE, axis_expand = 1.2, limits = TRUE, 
                      axes = TRUE, axislabels = TRUE) {
  
  if (!inherits(model_res, "rAMMI")) stop("Se requiere un objeto de clase 'rAMMI'")
  type <- label <- d1 <- d2 <- x0 <- y0 <- radio <- NULL

  scaling <- 0.5
  idx <- 1:model_res$Ncomp
  lambda_scaling <- model_res$eigenvalues[idx]^scaling
  
  coordgenotype <- t(t(model_res$gen_scores[, idx]) / lambda_scaling)
  coordenviroment <- t(t(model_res$env_scores[, idx]) * lambda_scaling)
  
  varexpl <- round((model_res$eigenvalues[idx]^2 / model_res$vartotal) * 100, 2)
  
  plotdata <- data.frame(
    rbind(
      data.frame(coordgenotype, type = "genotype", label = model_res$gen_labels),
      data.frame(coordenviroment, type = "environment", label = model_res$env_labels)
    )
  )
  colnames(plotdata)[1:2] <- c("Component1", "Component2")
  
  p <- ggplot(data = plotdata, aes(x = Component1, y = Component2)) +
    theme_classic()
  
  if (axes) {
    p <- p + geom_vline(xintercept = 0, linetype = "dashed", alpha = 0.5) +
      geom_hline(yintercept = 0, linetype = "dashed", alpha = 0.5)
  }
  
  p <- p + 
    geom_segment(data = subset(plotdata, type == "environment"),
                 aes(x = 0, y = 0, xend = Component1, yend = Component2),
                 arrow = arrow(length = unit(0.2, "cm")), col = colEnv, alpha = 0.7) +
    geom_text(data = subset(plotdata, type == "environment"),
              aes(label = label), col = colEnv, size = sizeEnv, vjust = -0.5) +
    geom_text(data = subset(plotdata, type == "genotype"),
              aes(label = label), col = colGen, size = sizeGen)
  
  if (titles) p <- p + ggtitle(paste("AMMI Biplot - Model:", model_res$type))
  
  if (axislabels) {
    p <- p + xlab(paste0("PC1 (", varexpl[1], "%)")) +
      ylab(paste0("PC2 (", varexpl[2], "%)"))
  }
  
  if (footnote) {
    foot_txt <- paste(model_res$type, "biplot explaining", sum(varexpl), "% of GxE variation")
    p <- p + labs(caption = foot_txt)
  }
  
  if (limits) p <- p + coord_fixed(ratio = 1)
  
  return(p)
}

#' GabrielEigen imputation method
#'
#' @param X a data frame or matrix with genotypes in rows and
#'   environments in columns when there are no replications of the
#'   experiment.
#' @references Arciniegas-Alarcón S., García-Peña M., Dias C.T.S., Krzanowski
#'   W.J. (2010). \emph{An alternative methodology for imputing missing data in
#'   trials with genotype-by-environment interaction}. Biometrical Letters 47,
#'   1–14.
#' @return A list containing:
#'   \itemize{
#'     \item NumeroIterGabriel: final number of iterations;
#'     \item CritConvergGabriel: maximum change of the estimated values for
#'     missing cells in the last step of iteration (precision of convergence);
#'     \item GabrielImput: imputed matrix (filled in with missing values
#'     estimated by the GabrielEigein procedure).
#'   }
#'
#' @keywords internal
#' @importFrom MASS ginv
#'

Gabriel.Calinski<-function(X){

  if (missing(X)) stop("Need to provide X data frame or matrix")
  stopifnot(
  class(X) %in% c("matrix", "data.frame")
  )


  nfilasX<-nrow(X)
  ncolX<-ncol(X)
  #nfilasX
  #ncolX
  if(ncolX>nfilasX) {X<-t(X)}

  XMISSING<-X
  is.matrix(X)
  totalelementos<-nfilasX*ncolX

  indicamissing<-is.na(XMISSING)
  indicadora<-indicamissing*1
  totalfaltantes<-sum(indicadora)
  medcol<-t(matrix(colMeans(XMISSING, na.rm=TRUE)))

  posicfaltantes<-matrix(0,totalfaltantes,2)
  indi3<-0
  for (indi1 in 1:nfilasX){
    for(indi2 in 1:ncolX){
      if (indicadora[indi1,indi2]==1){
        indi3<-indi3+1
        posicfaltantes[indi3,1]=indi1
        posicfaltantes[indi3,2]=indi2
        XMISSING[indi1,indi2]=medcol[1,indi2]
        #XMISSING[indi1,indi2]=MEDIA_inicial
      }
    }
  }
  #posicfaltantes
  #XMISSING

  completed_matrix<-XMISSING
  maxiter<-1000
  iter<-0
  desvest<-sqrt(t(diag(var(XMISSING))))
  #print(desvest)
  Xmissingestand<-scale(XMISSING)
  Prediction_Gabriel<-matrix(0,nfilasX,ncolX)
  epsilon<-1*(10**(-6))
  stabilitycrit<-1
  stabilityiter<-1


  while (stabilitycrit>epsilon & stabilityiter<=100){


    for (i1 in 1:totalfaltantes){

      posic_linha<-posicfaltantes[i1,1]
      posic_col<-posicfaltantes[i1,2]
      auxiliar1<-rbind(Xmissingestand[posic_linha,],Xmissingestand[-posic_linha,])
      auxiliar2<-cbind(auxiliar1[,posic_col],auxiliar1[,-posic_col])
      #print(i1)
      #print(auxiliar2)
      Y1puntoTRANS<-auxiliar2[1,2:ncolX]
      Ypunto1<-auxiliar2[2:nfilasX,1]
      Ybarra11<-auxiliar2[-1,-1]

      Ybarra11_Estand<-scale(Ybarra11)
      dvs_Ybarra11_Estand<-svd(Ybarra11_Estand)
      autovalores<-matrix(dvs_Ybarra11_Estand$d)**2
      sum_autovalores<-sum(autovalores)
      CritCalinski1<-(autovalores/sum_autovalores)
      CritCalinski2<-matrix(cumsum(CritCalinski1))
      CritCalinski3<-matrix(1,ncol(Ybarra11_Estand),1)
      for (Calins in 1:ncol(Ybarra11_Estand)){
        if (CritCalinski2[Calins,1]>0.70){CritCalinski3[Calins,1]=0}
      }
      f<-sum(CritCalinski3)+1
      #print(i1)
      #print(f)


      #print(Y1puntoTRANS)
      #print(Ypunto1)
      #print(Ybarra11)
      dvs.Ybarra11<-svd(Ybarra11)
      u_Ybarra11<-dvs.Ybarra11$u
      v_Ybarra11<-dvs.Ybarra11$v
      d_Ybarra11<-diag(dvs.Ybarra11$d)

      pred_estand<-Y1puntoTRANS%*%v_Ybarra11[,1:f]%*%MASS::ginv(d_Ybarra11[1:f,1:f])%*%t(u_Ybarra11[,1:f])%*%Ypunto1
      completed_matrix[posic_linha,posic_col]<-medcol[1,posic_col]+(pred_estand*desvest[1,posic_col])



    }

    stabilitycrit<-(sum((completed_matrix-XMISSING)**2))/(sum((XMISSING*indicadora)**2))
    stabilityiter<-stabilityiter+1
    XMISSING<-completed_matrix
    Xmissingestand<-scale(completed_matrix)
    medcol<-t(matrix(colMeans(completed_matrix)))
    desvest<-sqrt(t(diag(var(completed_matrix))))

    #print(stabilityiter)
    #print(stabilitycrit)
    ######print (f)
    #print(completed_matrix)



  }

  list(NumeroIterGabriel=stabilityiter,CritConvergGabriel=stabilitycrit,GabrielImput=completed_matrix)
}

#'Clones from the PLRV population
#'
#'@description  resistance study to PLRV (Patato Leaf Roll Virus) causing leaf
#'curl. 28 genotypes were experimented at 6 locations in Peru. Each clone was
#'evaluated three times in each environment, and yield, plant weight and plot were
#'registered.
#'
#'@name plrv
#'@docType data
#'@usage data(plrv)
#'
#'@format Data frame with 504 observations and 6 variables (genotype,
#'   locality, repetition, weightPlant, weightPlot and yield).
#'
#'@keywords datasets
#'
#'@references Felipe de Mendiburu (2020). agricolae: Statistical Procedures for
#'  Agricultural Research. R package version 1.3-2.
#'  \url{https://CRAN.R-project.org/package=agricolae}
#'
#' @examples
#' library(geneticae)
#' data(plrv)
#' str(plrv)
#'
NULL

#' Eigenvector Imputation Function (Internal)
#' 
#' Internal function for GxE imputation using the Krzanowski (1988) eigenvector 
#' approach with a leave-one-out strategy.
#'
#' @param X A matrix with missing values (NAs).
#' @param f Number of components (rank) to use for the reconstruction.
#' @return A list with the number of iterations, convergence status, 
#' final rank used, and the imputed matrix.
#' @importFrom MASS ginv
#' @importFrom corpcor fast.svd
#' @importFrom stats var
#' @keywords internal

Eigenvectorfun <- function(X, f) {
  
  X 
  is.matrix(X) 
  nfilasX<-nrow(X) 
  ncolX<-ncol(X)
  nfilasX
  ncolX
  totalelementos<-nfilasX*ncolX
  
  # Mode <- function(x) {
  #   ux <- unique(x)
  #   ux[which.max(tabulate(match(x, ux)))]
  # }
  # bestrank=numeric(1000)
  # for(i in 1:1000){
  #   koptimalmatrix <- summary(cv.svd.wold(X))
  #   bestrank[i] <- koptimalmatrix$rank.best
  # }
  # #table(bestrank)
  # f<-Mode(bestrank)
  if (f==ncol(X)) {f<-f-1}
  
  

  XMISSING=X
  indicamissing<-is.na(XMISSING)
  indicadora<-indicamissing*1
  totalfaltantes<-sum(indicadora)
  medcol<-t(matrix(colMeans(XMISSING, na.rm=TRUE))) 

  incompleta<-XMISSING
  XMISSING
  posicfaltantes<-matrix(0,totalfaltantes,2)
  indi3<-0
  for (indi1 in 1:nfilasX){
    for(indi2 in 1:ncolX){
      if (indicadora[indi1,indi2]==1){
        indi3<-indi3+1
        posicfaltantes[indi3,1]=indi1
        posicfaltantes[indi3,2]=indi2
        XMISSING[indi1,indi2]=medcol[1,indi2]
        
      } 
    }
  } 
  
  posicfaltantes
  XMISSING
  completed_matrix<-XMISSING
  maxiter<-50
  iter<-0
  desvest<-sqrt(t(diag(var(XMISSING))))
  #print(desvest)
  Xmissingestand<-scale(XMISSING)
  epsilon<-1*(10**(-2))
  stabilitycrit<-1
  stabilityiter<-0
  
  while (stabilitycrit>epsilon & stabilityiter<=500){
    
    for (i in 1:totalfaltantes){
      
      oneout<-Xmissingestand[-posicfaltantes[i,1],]
      dvs_oneout<-fast.svd(oneout)
      
      if(f==1){
        u<-dvs_oneout$u[,f]
        d<-diag(dvs_oneout$d)
        v<-dvs_oneout$v[,f]
        T<-matrix(u*d[f,f])
        P<-matrix(v)
      }
      
      else{
        u<-dvs_oneout$u[,1:f]
        d<-diag(dvs_oneout$d)
        v<-dvs_oneout$v[,1:f]
        T<-u%*%d[1:f,1:f]
        P<-v
      }
      
      tj_T<-t(Xmissingestand[posicfaltantes[i,1],-posicfaltantes[i,2]])%*%P[-posicfaltantes[i,2],]%*%ginv(t(P[-posicfaltantes[i,2],])%*%P[-posicfaltantes[i,2],])
      x_ij<-tj_T%*%P[posicfaltantes[i,2],]
      completed_matrix[posicfaltantes[i,1],posicfaltantes[i,2]]<-medcol[1,posicfaltantes[i,2]]+(x_ij*desvest[1,posicfaltantes[i,2]])

      
    } 
    
    convergence.old<-stabilitycrit
    stabilitycrit1<-sqrt(  (1/totalfaltantes)*(sum((completed_matrix-XMISSING)**2))  )
    stabilitycrit2<-sqrt( (1/(totalelementos-totalfaltantes))*(sum((XMISSING*(1-indicadora))**2))  )
    stabilitycrit<-stabilitycrit1/stabilitycrit2
    stabilityiter<-stabilityiter+1
    XMISSING<-completed_matrix
    Xmissingestand<-scale(completed_matrix)
    medcol<-t(matrix(colMeans(completed_matrix)))
    desvest<-sqrt(t(diag(var(completed_matrix))))
    convergence.new<-stabilitycrit

    
    if(f!=1 & convergence.new>convergence.old){
      f<-f-1
      stabilitycrit<-1
      stabilityiter<-0
    }
    
    
  }
  
  list(EigenItera=stabilityiter, EigenConverg=stabilitycrit, EigenComp=f, EigenImputaciones=completed_matrix)
  
}

if (getRversion() >= "2.15.1")  {
  utils::globalVariables(
    c("X1", "X2" ,"end" ,"label", "radio", "start", "x0" ,"x1_x", "x1_y" ,"y0","value", "variable", "genotype", "Data", "Component1", "Component2","env", "mean_resp")
  )
}

#'GGE biplots with \pkg{ggplot2}
#'
#' @description GGE biplots are used for visual examination of the relationships
#'  between test environments, genotypes, and genotype-by-environment
#'  interactions. `rSREGPlot()` produces a biplot as an object of class 'ggplot',
#'  using the output of the \code{\link{rSREGModel}} function.
#'  Several types of biplots are offered which focus on different aspects of the
#'  analysis. Customization options are also included. This function is a
#'  modification of the `rSREGPlot` function from the 
#'  \href{https://CRAN.R-project.org/package=GGEBiplots}{GGEBiplots package}.
#'
#'@param rSREGModel An object of class \code{rSREGModel}.
#'@param type type of biplot to produce.
#'\itemize{
#'  \item \code{"Biplot"}: Basic biplot.
#'  \item \code{"Selected Environment"}: Ranking of cultivars based on
#'  their performance in any given environment.
#'  \item \code{"Selected Genotype"}: Ranking of environments based on the
#'  performance of any given cultivar.
#'  \item \code{"Relationship Among Environments"}.
#'  \item \code{"Comparison of Genotype"}.
#'  \item \code{"Which Won Where/What"}: Identifying the 'best'
#'  cultivar in each environment.
#'  \item \code{"Discrimination vs. representativeness"}: Evaluating the
#'  environments based on both discriminating ability and representativeness.
#'  \item \code{"Ranking Environments"}: Ranking environments with respect to
#'  the ideal environment.
#'  \item \code{"Mean vs. stability"}: Evaluating cultivars based on both average
#'  yield and stability.
#'  \item \code{"Ranking Genotypes"}: Ranking genotypes with respect to the
#'  ideal genotype.}
#'@param d1 PCA component to plot on x axis. Defaults to 1.
#'@param d2 PCA component to plot on y axis. Defaults to 2.
#'@param selectedE name of the environment to evaluate when `type="Selected
#'  Environment"`.
#'@param selectedG name of the genotype to evaluate when `type="Selected
#'  Genotype"`.
#'@param selectedG1, name of the genotype to compare to `selectedG2`  when
#'  `type="Comparison of Genotype"`.
#'@param selectedG2, name of the genotype to compare to `selectedG1`  when
#'  `type="Comparison of Genotype"`.
#'@param colGen genotype attributes colour. Defaults to `"gray47"`.
#'@param colEnv environment attributes colour. Defaults to `"darkred"`.
#'@param colSegment segment or circle lines colour. Defaults to `"gray30"`.
#'@param colHull hull colour when `type="Which Won Where/What"`. Defaults to
#'  "gray30".
#'@param sizeGen genotype labels text size. Defaults to 4.
#'@param sizeEnv environment labels text size. Defaults to 4.
#'@param largeSize larger labels text size to use for two selected genotypes in
#'  `type="Comparison of Genotype"`, and for the outermost genotypes in
#'  `type="Which Won Where/What"`. Defaults to 4.5.
#'@param axis_expand multiplication factor to expand the axis limits by to
#'  enable fitting of labels. Defaults to 1.2.
#'@param axislabels logical, if this argument is `TRUE` labels for axes are
#'  included. Defaults to `TRUE`.
#'@param axes logical, if this argument is `TRUE` x and y axes going through the
#'  origin are drawn. Defaults to `TRUE`.
#'@param limits logical, if this argument is `TRUE` the axes are re-scaled.
#'  Defaults to `TRUE`.
#'@param titles logical, if this argument is `TRUE` a plot title is included.
#'  Defaults to `TRUE`.
#'@param footnote logical, if this argument is `TRUE` a footnote is included.
#'  Defaults to `TRUE`.
#'@keywords GGE Biplot
#'@return A biplot of class \code{ggplot}
#'@references Yan W, Kang M (2003). \emph{GGE Biplot Analysis: A Graphical Tool
#'  for Breeders, Geneticists, and Agronomists}. CRC Press.
#'@references Sam Dumble (2017). GGEBiplots: GGE Biplots with 'ggplot2'. R
#'  package version 0.1.1. \url{https://CRAN.R-project.org/package=GGEBiplots}
#'@export
#'@examples
#'  library(geneticae)
#'
#'  # Data without replication
#'  library(agridat)
#'  data(yan.winterwheat)
#'  GGE1 <- rSREGModel(yan.winterwheat)
#'  rSREGPlot(GGE1, sizeGen=4, sizeEnv=4)
#'
#'  # Data with replication
#'  data(plrv)
#'  GGE2 <- rSREGModel(plrv, genotype = "Genotype", environment = "Locality",
#'                   response = "Yield", rep = "Rep")
#'  rSREGPlot(GGE2, sizeGen=4, sizeEnv=4)
#'
#'@importFrom ggplot2 aes arrow coord_fixed element_text geom_abline geom_hline
#'  geom_point geom_polygon geom_segment geom_text geom_vline ggplot ggtitle
#'  labs layer_scales scale_color_manual scale_size_manual scale_x_continuous
#'  scale_y_continuous theme theme_bw xlab ylab element_blank theme_classic
#'@importFrom ggforce geom_arc geom_circle
#'@importFrom scales alpha squish
#'@importFrom grDevices chull
#'@importFrom grid unit

rSREGPlot<-function(rSREGModel,type="Biplot",d1=1,d2=2, selectedE=NA , selectedG=NA,selectedG1=NA,selectedG2=NA,
                  colGen="gray47",colEnv="darkred",colSegment="gray30",colHull="gray30",sizeGen=6,sizeEnv=6,largeSize=4.5,axis_expand=1.2,
                  axislabels=TRUE,axes=TRUE,limits=TRUE,titles=TRUE,footnote=TRUE){
  
  stopifnot(
    class(rSREGModel) == "rSREGModel",
    type %in% c("Biplot", "Selected Environment","Selected Genotype","Relationship Among Environments",
                "Comparison of Genotype","Which Won Where/What","Discrimination vs. representativeness",
                "Ranking Environments","Mean vs. Stability","Ranking Genotypes"),
    class(d1) == "numeric",
    class(d2) == "numeric",
    class(colGen) == "character",
    class(colEnv) == "character",
    class(colSegment) == "character",
    class(colHull) == "character",
    class(sizeGen) == "numeric",
    class(sizeEnv) == "numeric",
    class(largeSize) == "numeric",
    class(axis_expand) == "numeric",
    class(axislabels)  == "logical",
    class(axes)  == "logical",
    class(limits)  == "logical",
    class(titles)  == "logical",
    class(footnote)  == "logical"
  )
  
  coordgenotype=rSREGModel$coordgenotype[,c(d1,d2)];coordenviroment=rSREGModel$coordenviroment[,c(d1,d2)]
  varexpl=rSREGModel$varexpl[c(d1,d2)];labelgen=rSREGModel$labelgen;
  labelenv=rSREGModel$labelenv;labelaxes=rSREGModel$labelaxes[c(d1,d2)];Data=rSREGModel$Data
  SVP=rSREGModel$SVP
  
  plotdata<-data.frame(rbind(data.frame(coordgenotype,type="genotype",label=labelgen),data.frame(coordenviroment,type="environment",label=labelenv)))
  colnames(plotdata)[1:2]<-c("d1","d2")
  plotdata$type<-factor(plotdata$type)
  
  
  GGE1<-ggplot(data=plotdata,aes(x=d1,y=d2,group="type"))+
    theme(panel.grid = element_blank())+
    scale_color_manual(values=c(colGen,colEnv)) +
    scale_size_manual(values=c(sizeGen,sizeEnv)) + theme_classic()
  
  labelaxes_x <- paste("PC", d1) 
  labelaxes_y <- paste("PC", d2)
  
  if(axislabels == TRUE){
    GGE1 <- GGE1 + 
      xlab(paste(labelaxes_x, " (", format(varexpl[1], nsmall = 2), "%)", sep = "")) +
      ylab(paste(labelaxes_y, " (", format(varexpl[2], nsmall = 2), "%)", sep = "")) +
      theme(
        axis.title = element_text(size = 16),
        axis.text = element_text(size = 14)
      )
  } else {
    GGE1 <- GGE1 + xlab(" ") + ylab(" ")
  }
  
  if(axes==TRUE){
    GGE1<-GGE1+geom_hline(yintercept=0)+geom_vline(xintercept=0)
  }
  if(limits==TRUE){
    xlim<-c(min(plotdata$d1*axis_expand),max(plotdata$d1*axis_expand))
    ylim<-c(min(plotdata$d2*axis_expand),max(plotdata$d2*axis_expand))
    if(which(c(diff(xlim),diff(ylim))==max(c(diff(xlim),diff(ylim))))==1){
      xlim1<-xlim
      ylim1<-c(ylim[1]-(diff(xlim)-diff(ylim))/2,ylim[2]+(diff(xlim)-diff(ylim))/2)
    }
    if(which(c(diff(xlim),diff(ylim))==max(c(diff(xlim),diff(ylim))))==2){
      ylim1<-ylim
      xlim1<-c(xlim[1]-(diff(ylim)-diff(xlim))/2,xlim[2]+(diff(ylim)-diff(xlim))/2)
    }
    
    GGE1<-GGE1+scale_x_continuous(limits=xlim1,expand=c(0,0))+
      scale_y_continuous(limits=ylim1,expand=c(0,0))+
      coord_fixed(1)
  }
  
  
  
  
  if(type=="Biplot"){
    
    GGE2<-GGE1+geom_segment(xend=0,yend=0,col=alpha(colEnv,0.5),data=subset(plotdata,type=="environment"))+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="genotype"),col=colGen,size=sizeGen)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="environment"),col=colEnv,size=sizeEnv)
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle("GGE Biplot")}
  }
  
  if(type=="Selected Environment"){
    if(!selectedE%in%labelenv){stop(paste(selectedE,"The environment selected is not in list of environment labels"))}
    venvironment<-labelenv==selectedE
    
    x1<-NULL
    for (i in 1:nrow(Data))
    {
      x <- solve(matrix(c(-coordenviroment[venvironment,2], coordenviroment[venvironment,1], coordenviroment[venvironment,1], coordenviroment[venvironment, 2]), nrow = 2),
                 matrix(c(0, coordenviroment[venvironment,1] * coordgenotype[i, 1] +  coordenviroment[venvironment, 2] *coordgenotype[i, 2]), ncol = 1))
      
      x1<-rbind(x1,t(x))
    }
    
    plotdata$x1_x<-NA
    plotdata$x1_x[plotdata$type=="genotype"]<-x1[,1]
    
    plotdata$x1_y<-NA
    plotdata$x1_y[plotdata$type=="genotype"]<-x1[,2]
    
    GGE2<-GGE1+
      geom_abline(slope=coordenviroment[venvironment, 2]/coordenviroment[venvironment,1],intercept=0,col=colEnv,)+
      geom_abline(slope=-coordenviroment[venvironment, 1]/coordenviroment[venvironment,2],intercept=0,col=colEnv)+
      geom_segment(xend=0,yend=0,col=alpha(colEnv,0.5),data=subset(plotdata,type=="environment"&label==selectedE),
                   arrow =arrow(ends ="first",length=unit(0.2,"inches") ),linewidth=1)+
      geom_segment(aes(xend=x1_x,yend=x1_y),col=colGen,data=subset(plotdata,type=="genotype"),linetype=2)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="genotype"),col=colGen,size=sizeGen)
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle(paste("Selected Environment =",selectedE))}
    
  }
  
  
  if(type=="Selected Genotype"){
    if(!selectedG%in%labelgen){stop(paste(selectedG,"The genotype selected is not in list of genotype labels"))}
    vgenotype<-labelgen==selectedG
    
    x1<-NULL
    for (i in 1:ncol(Data))
    {
      x <- solve(matrix(c(-coordgenotype[vgenotype,2], coordgenotype[vgenotype,1], coordgenotype[vgenotype,1], coordgenotype[vgenotype, 2]), nrow = 2),
                 matrix(c(0, coordgenotype[vgenotype,1] * coordenviroment[i, 1] +  coordgenotype[vgenotype, 2] *coordenviroment[i, 2]), ncol = 1))
      
      x1<-rbind(x1,t(x))
    }
    
    plotdata$x1_x<-NA
    plotdata$x1_x[plotdata$type=="environment"]<-x1[,1]
    
    plotdata$x1_y<-NA
    plotdata$x1_y[plotdata$type=="environment"]<-x1[,2]
    
    GGE2<-GGE1+
      geom_abline(slope=coordgenotype[vgenotype, 2]/coordgenotype[vgenotype,1],intercept=0,col=alpha(colSegment,0.5),linewidth=0.7)+
      geom_abline(slope=-coordgenotype[vgenotype, 1]/coordgenotype[vgenotype,2],intercept=0,col=alpha(colSegment,0.5),linewidth=0.7)+
      geom_segment(xend=0,yend=0,col=alpha(colSegment,0.5),data=subset(plotdata,type=="genotype"&label==selectedG),
                   arrow =arrow(ends ="first",length=unit(0.15,"inches") ),linewidth=1)+
      geom_segment(aes(xend=x1_x,yend=x1_y),col=colEnv,data=subset(plotdata,type=="environment"),linetype=2)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="environment"),col=colEnv,size=sizeEnv)
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle(paste("Selected Genotype =",selectedG))}
  }
  
  if(type=="Relationship Among Environments"){
    
    
    GGE2<-GGE1+
      geom_segment(xend=0,yend=0,col=alpha(colEnv,0.5),data=subset(plotdata,type=="environment"),
                   arrow =arrow(ends ="first",length=unit(0.1,"inches") ),linewidth=0.7, linetype =2)+
      geom_text(aes(label=label),show.legend = FALSE,hjust="outward",vjust="outward",data=subset(plotdata,type=="environment"),col=colEnv,size=sizeEnv)
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle("Relationship Among Environments")}
  }
  
  if(type=="Comparison of Genotype"){
    
    if(!selectedG1%in%labelgen){stop(paste(selectedG1,"The genotype selected is not in list of genotype labels"))}
    if(!selectedG2%in%labelgen){stop(paste(selectedG2,"The genotype selected is not in list of genotype labels"))}
    if(selectedG1==selectedG2){stop(paste("Cannot compare the same genotype to itself"))}
    vgenotype1<-labelgen==selectedG1
    vgenotype2<-labelgen==selectedG2
    
    # Compara dos genotipos
    GGE2<-GGE1+geom_point(data=subset(plotdata,type=="genotype"&label%in%c(selectedG1,selectedG2)))+
      annotate("segment", 
               x = plotdata$d1[plotdata$label==selectedG1 & plotdata$type=="genotype"],
               xend = plotdata$d1[plotdata$label==selectedG2 & plotdata$type=="genotype"],
               y = plotdata$d2[plotdata$label==selectedG1 & plotdata$type=="genotype"],
               yend = plotdata$d2[plotdata$label==selectedG2 & plotdata$type=="genotype"], 
               col=colGen, linewidth=1) + 
    geom_abline(intercept = 0, slope = -(coordgenotype[vgenotype1, 1] - coordgenotype[vgenotype2, 1])/(coordgenotype[vgenotype1,2] - coordgenotype[vgenotype2, 2]),
                col = colGen, linewidth=0.7)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="environment"),col=colEnv,size=sizeEnv)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="genotype"&!label%in%c(selectedG1,selectedG2)),col=alpha(colGen,0.4),size=sizeGen)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="genotype"&label%in%c(selectedG1,selectedG2)),
                col=colGen,size=largeSize,hjust="outward",vjust="outward")
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle(paste("Comparison of Genotype",selectedG1,"with Genotype",selectedG2))}
    
  }
  
  if(type=="Which Won Where/What"){
    # Which-won-where
    indice = c(grDevices::chull(coordgenotype[, 1], coordgenotype[,2]))
    www<-data.frame(coordgenotype[indice,])
    
    
    indice<-c(indice,indice[1])
    ######################################
    segs<-NULL
    
    
    limx<-layer_scales(GGE1)$x$limits
    limy<-layer_scales(GGE1)$y$limits
    
    i <- 1
    while (is.na(indice[i + 1]) == FALSE)
    {
      m<-(coordgenotype[indice[i], 2] - coordgenotype[indice[i + 1], 2])/(coordgenotype[indice[i],1]-coordgenotype[indice[i + 1],1])
      mperp<--1/m
      c2<-coordgenotype[indice[i + 1], 2] - m*coordgenotype[indice[i + 1],1]
      xint<--c2/(m-mperp)
      xint<-ifelse(xint<0,min(coordenviroment[, 1],coordgenotype[, 1]), max(coordenviroment[, 1],coordgenotype[, 1]))
      yint<-mperp*xint
      
      xprop<-ifelse(xint<0,xint/limx[1],xint/limx[2])
      yprop<-ifelse(yint<0,yint/limy[1],yint/limy[2])
      
      m3<-which(c(xprop,yprop)==max(c(xprop,yprop)))
      m2<-abs(c(xint,yint)[m3])
      if(m3==1&xint<0)sl1<-(c(xint,yint)/m2)*abs(limx[1])
      if(m3==1&xint>0)sl1<-(c(xint,yint)/m2)*limx[2]
      if(m3==2&yint<0)sl1<-(c(xint,yint)/m2)*abs(limy[1])
      if(m3==2&yint>0)sl1<-(c(xint,yint)/m2)*limy[2]
      
      segs<-rbind(segs,sl1)
      i <- i + 1
    }
    rownames(segs)<-NULL
    colnames(segs)<-NULL
    segs<-data.frame(segs)
    
    
    
    colnames(www)<-c("X1","X2")
    
    winners<-plotdata[plotdata$type=="genotype",][indice[-1],]
    others<-plotdata[!rownames(plotdata)%in%rownames(winners),]
    
    GGE2<-GGE1+geom_polygon(aes(x=X1,y=X2),data=www,fill=NA,col=colHull,linewidth=0.7)+
      geom_segment(data=segs,aes(x=X1,y=X2),xend=0,yend=0,linetype=2,col=colSegment,linewidth=0.7) +
      geom_text(aes(label=label),show.legend = FALSE,data=subset(others,type=="genotype"),col=colGen,size=sizeGen)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(others,type=="environment"),col=colEnv,size=sizeEnv)+
      geom_text(aes(label=label,x=d1,y=d2),show.legend = FALSE,hjust="outward",vjust="outward",data=winners,col=colGen,size=largeSize,fontface="bold")
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle("Which Won Where/What")}
    
  }
  
  if(type=="Discrimination vs. representativeness"){
    
    circles<-data.frame(x0=0,y0=0,start=0,end=pi*2,radio =1:5* max((max(coordenviroment[1, ]) - min(coordenviroment[1, ])), (max(coordenviroment[2,]) - min(coordenviroment[2, ])))/10)
    
    
    GGE2<-GGE1+geom_arc(aes(r=radio,x0=x0,y0=y0,start=start,end=end),data=circles,col=colSegment,inherit.aes=F, na.rm=TRUE,linetype =2)+
      geom_segment(xend=0,yend=0,x=mean(coordenviroment[,1]),y=mean(coordenviroment[,2]),arrow =arrow(ends ="first",length=unit(0.1,"inches") ),linewidth=1,col=colSegment)+
      geom_abline(intercept=0,slope=mean(coordenviroment[, 2])/mean(coordenviroment[,1]),col=colSegment)+
      geom_segment(xend=0,yend=0,col=alpha(colEnv,0.5),data=subset(plotdata,type=="environment"),linetype=2)+
      geom_text(aes(col=type,label=label,size=type),show.legend = FALSE)
    
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle("Discrimination vs. representativeness")}
    
  }
  
  if(type=="Ranking Environments"){
    
    med1 = mean(coordenviroment[, 1])
    med2 = mean(coordenviroment[, 2])
    mod = max((coordenviroment[, 1]^2 + coordenviroment[,2]^2)^0.5)
    xcoord = sign(med1) * (mod^2/(1 + med2^2/med1^2))^0.5
    ycoord = (med2/med1) * xcoord
    
    circles<-data.frame(x0=xcoord,y0=ycoord,start=0,end=pi*2,radio=1:8*((xcoord - med1)^2 + (ycoord - med2)^2)^0.5/3)
    
    GGE2<-GGE1+
      geom_abline(intercept=0, slope=med2/med1, col=colSegment, linewidth=0.7)+
      geom_abline(intercept=0, slope=-med1/med2, col=colSegment, linewidth=0.7)+
      geom_arc(aes(r=radio, x0=x0, y0=y0, start=start, end=end), 
               data=circles, col=colSegment, inherit.aes=F, na.rm=TRUE, linetype=2)+
      annotate("segment", x=0, y=0, xend=xcoord, yend=ycoord, 
               arrow = arrow(length=unit(0.15,"inches")), 
               linewidth=1, col=colSegment)+
    geom_text(aes(label=label), show.legend = FALSE, 
              data=subset(plotdata, type=="environment"), col=colEnv, size=sizeEnv)
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle("Ranking Environments")}
  }
  
  
  if(type=="Mean vs. Stability"){
    
    med1 = mean(coordenviroment[, 1])
    med2 = mean(coordenviroment[, 2])
    
    x1<-NULL
    
    for (i in 1:nrow(Data))
    {
      x <- solve(matrix(c(-med2, med1, med1, med2), nrow = 2), matrix(c(0, med2 * coordgenotype[i, 2] + med1 * coordgenotype[i, 1]),ncol = 1))
      x1<-rbind(x1,t(x))
      
    }
    plotdata$x1_x<-NA
    plotdata$x1_x[plotdata$type=="genotype"]<-x1[,1]
    
    plotdata$x1_y<-NA
    plotdata$x1_y[plotdata$type=="genotype"]<-x1[,2]
    
    GGE2<-GGE1+
      geom_abline(intercept=0,slope=med2/med1,col=colSegment, linewidth=0.7)+
      geom_abline(intercept=0,slope=-med1/med2,col=colSegment, linewidth=0.7)+
      geom_segment(aes(xend=x1_x, yend=x1_y),col=colGen,linetype=2,data=subset(plotdata,type=="genotype"))+
      annotate("segment", x=0, y=0, xend=med1, yend=med2, 
               arrow = arrow(length=unit(0.15,"inches")), 
               linewidth=1, col=colSegment) +
    geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="genotype"),col=colGen,size=sizeGen)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="environment"),col=colEnv,size=sizeEnv)
    
    if(titles==TRUE){GGE2<-GGE2+ggtitle("Mean vs. Stability")}
  }
  
  
  if(type=="Ranking Genotypes"){
    
    med1 = mean(coordenviroment[, 1])
    med2 = mean(coordenviroment[, 2])
    coordx <- 0
    coordy <- 0
    for (i in 1:nrow(Data)) {
      x <- solve(matrix(c(-med2, med1, med1, med2),nrow = 2), matrix(c(0, med2 * coordgenotype[i,2] + med1 * coordgenotype[i, 1]),ncol = 1))
      if (sign(x[1]) == sign(med1)) {
        if (abs(x[1]) > abs(coordx)) {
          coordx <- x[1]
          coordy <- x[2]
        }
      }
    }
    
    
    circles<-data.frame(x0=coordx,y0=coordy,radio=1:10*((coordx - med1)^2 + (coordy - med2)^2)^0.5/3)
    

    GGE2<-GGE1+
      geom_abline(intercept=0,slope=med2/med1,col=colGen, linewidth=0.7)+
      geom_abline(intercept=0,slope=-med1/med2,col=colGen, linewidth=0.7)+
      geom_arc(aes(r=radio,x0=x0,y0=y0,start=0,end=pi*2),data=circles,col=colSegment,inherit.aes=F,linetype =2, na.rm=TRUE)+
      annotate("segment", x=0, y=0, xend=coordx, yend=coordy, 
               arrow = arrow(length=unit(0.15,"inches")), 
               linewidth=1, col=colGen) +
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="genotype"),col=colGen,size=sizeGen)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="environment"),col=colEnv,size=sizeEnv)
    
    if(limits==F){
      GGE2<-GGE2+ scale_x_continuous(limits=c(min(plotdata$d1*axis_expand),max(plotdata$d1*axis_expand)),oob=squish)+
        scale_y_continuous(limits=c(min(plotdata$d2*axis_expand),max(plotdata$d2*axis_expand)),oob=squish)
    }
    if(axes==TRUE){
      GGE2<-GGE2+geom_hline(yintercept=0)+geom_vline(xintercept=0)
    }
    GGE2<-GGE2+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="genotype"),col=colGen,size=sizeGen)+
      geom_text(aes(label=label),show.legend = FALSE,data=subset(plotdata,type=="environment"),col=colEnv,size=sizeEnv)
    if(titles==TRUE){GGE2<-GGE2+ggtitle("Ranking Genotypes")}
    
    
  }
  
  if(footnote==TRUE){
    SVPtext<-ifelse(SVP==1|SVP=="row","Row Metric Preserving SVP",
                    ifelse(SVP==2|SVP=="column","Column Metric Preserving SVP",
                           ifelse(SVP==3|SVP=="dual","Dual Metric Preserving SVP",
                                  ifelse(SVP==4|SVP=="symmetrical","Symmetrical SVP","NIPALS algorithm"))))
    footnotetxt=paste("\nGGE Biplot showing components ",d1," and ",d2," explaining ",sum(varexpl),"% of the total variation\nusing ",
                      SVPtext,sep="")
    
    GGE2<-GGE2+ labs(caption = footnotetxt)+theme(plot.caption = element_text(size=10,hjust=0,face="italic"))
  }
  
  
  
  return(GGE2)
}

library(testthat)
library(geneticae)
library(ggplot2)

# --- FUNCIÓN AUXILIAR PARA UN MODELO ESTABLE ---
generar_modelo_gge <- function() {
  set.seed(123)
  df <- expand.grid(Genotype = paste0("G", 1:6), 
                    Locality = paste0("L", 1:4)) %>%
    mutate(Yield = 10 + rnorm(n()))
  # rSREGModel es el motor para GGE
  return(rSREGModel(df, "Genotype", "Locality", "Yield"))
}

test_that("rSREGPlot genera todos los tipos de biplot sin errores", {
  gge_obj <- generar_modelo_gge()
  
  # Tipos que no requieren parámetros adicionales
  tipos_simples <- c("Biplot", "Relationship Among Environments", 
                     "Which Won Where/What", "Discrimination vs. representativeness",
                     "Ranking Environments", "Mean vs. Stability", "Ranking Genotypes")
  
  for (t in tipos_simples) {
    p <- rSREGPlot(gge_obj, type = t)
    expect_s3_class(p, "ggplot")
    expect_true(length(p$layers) > 0)
  }
}) #

#'Site Regression model
#'
#'@description The Site Regression model (also called genotype +
#'  genotype-by-environment (GGE) model) is a powerful tool for effective
#'  analysis and interpretation of data from multi-environment trials in
#'  breeding programs. There are different functions in R to fit the SREG model,
#'  however, this function has the following improvements: \itemize{ \item
#'  Includes recently published robust versions of the SREG model (Angelini et
#'  al., 2022). \item It can be used for data from trials with repetitions
#'  (there is no need to calculate means beforehand). \item Other variables not
#'  used in the analysis can be present in the dataset.}
#'
#'@param Data dataframe with genotypes, environments, repetitions (if any) and
#'  the phenotypic trait of interest. Additional variables that will not be used
#'  in the model may be present in the data.
#'@param genotype column name for genotypes.
#'@param environment column name for environments.
#'@param response column name for the phenotypic trait.
#'@param rep column name for replications. If this argument is NULL, there are
#'  no replications in the data. Defaults to NULL.
#'@param model method for fitting the SREG model: `"SREG"`,`"CovSREG"`,`"hSREG"`
#'  or `"ppSREG"` (see References). Defaults to `"SREG"`.
#'@param SVP method for singular value partitioning. Either `"row"`, `"column"`,
#'  or `"symmetrical"`. Defaults to `"symmetrical"`.
#'@return A list of class \code{GGE_Model} containing: \item{model}{SREG model
#'  version.} \item{coordgenotype}{plotting coordinates for each genotype in
#'  every component.} \item{coordenviroment}{plotting coordinates for each
#'  environment in every component.} \item{eigenvalues}{vector of eigenvalues
#'  for each component.} \item{vartotal}{overall variance.}
#'  \item{varexpl}{percentage of variance explained by each component.}
#'  \item{labelgen}{genotype names.} \item{labelenv}{environment names.}
#'  \item{axes}{axis labels.} \item{Data}{scaled and centered input data.}
#'  \item{SVP}{name of SVP method.}
#'
#'@return A biplot of class \code{ggplot}
#'
#'@details A linear model by robust regression using an M estimator proposed by
#'  Huber (1964, 1973) fitted by iterated re-weighted least squares, in
#'  combination with three robust SVD/PCA procedures, resulted in a total of
#'  three robust SREG alternatives. The robust SVD/PCA considered were:
#'  \itemize{ \item CovSREG: robust PCA that is obtained by replacing the
#'  classical estimates of location and covariance by their robust analogues
#'  using Minimum Regularized Covariance Determinant (MRCD) approach; \item
#'  hSREG: robust PCA method that tries to combine the advantages of both
#'  approaches, PCA based on a robust covariance matrix and based on projection
#'  pursuit; \item ppSREG: robust PCA that uses the projection pursuit and
#'  directly calculates the robust estimates of the eigenvalues and eigenvectors
#'  without going through robust covariance estimation. It is a very attractive
#'  method for bigdata situations, which are very common in METs (a few
#'  genotypes tested in a large number of environments), as the principal
#'  components can be calculated sequentially. }
#'@references  Julia Angelini, Gabriela Faviere, Eugenia Bortolotto, Gerardo
#'  Domingo Lucio Cervigni & Marta Beatriz Quaglino (2022) Handling outliers in
#'  multi-environment trial data analysis: in the direction of robust SREG
#'  model, Journal of Crop Improvement, \doi{10.1080/15427528.2022.2051217}
#'  
#'@export
#'@examples
#'  library(geneticae)
#'
#'  # Data without replication
#'  library(agridat)
#'  data(yan.winterwheat)
#'  GGE1 <- rSREGModel(yan.winterwheat, genotype="gen", environment="env", response="yield")
#'
#'  # Data with replication
#'  data(plrv)
#'  GGE2 <- rSREGModel(plrv, genotype = "Genotype", environment = "Locality",
#'                   response = "Yield", rep = "Rep")
#'
#'@importFrom MASS rlm
#'@importFrom pcaMethods robustSvd
#'@importFrom rrcov PcaHubert PcaCov PcaProj CovControlMrcd getEigenvalues
#'  getLoadings getScores
#'@importFrom stats lm residuals
#'@importFrom dplyr group_by summarise rename pull %>%
#'@importFrom rlang sym

rSREGModel<- function(Data, genotype = "gen", environment = "env", response = "yield",
                    rep=NULL, model = "SREG", SVP="symmetrical")
{
  
  if (missing(Data)) stop("Need to provide Data data frame")
  if(any(is.na(Data))){stop("Missing data in input data frame, run the imputation function first to complete the data set")}
  stopifnot(
    class(Data) == "data.frame",
    class(genotype) == "character",
    class(environment) == "character",
    class(response) == "character",
    class(rep)%in% c("character", "NULL"),
    model %in% c("SREG", "hSREG", "ppSREG", "CovSREG"),
    SVP %in% c("symmetrical", "row", "column")
  )
  
  if(!is.null(rep)){
    Data <-
      Data %>%
      group_by(!!sym(environment), !!sym(genotype)) %>%
      summarise(y=mean(!!sym(response)))
    response = "y"
  }
  
  gen <- as.factor(pull(Data, genotype))
  env <- as.factor(pull(Data, environment))
  y <- pull(Data, response)
  
  Ngen <- nlevels(gen)
  Nenv <- nlevels(env)
  labelgen <- unique(gen)
  labelenv <- unique(env)
  Ngen <- nlevels(gen)
  Nenv <- nlevels(env)
  Max <- min(Ngen - 1, Nenv)
  
  if (model == "SREG"){
    lm.x<- lm(y ~ env, data=Data, contrasts=list(env="contr.sum"))
    residuals.x<- matrix(residuals(lm.x), nrow = Ngen , ncol = Nenv)
    svd.x<- svd(residuals.x)
    singlevalue<-svd.x$d
    eigenvalues = singlevalue^2
    loading<-(svd.x$v)
    score<-(svd.x$u%*%diag(svd.x$d))
  }
  #
  else if (model == "CovSREG"){
    rlm.x<- rlm(y~env, data=Data, contrasts=list(env="contr.sum"),maxit = 50)
    residuals.x<- matrix(residuals(rlm.x), nrow = Ngen, ncol = Nenv)
    
    PcaCov <- PcaCov(residuals.x, cov.control=CovControlMrcd())
    
    loading<- getLoadings(PcaCov)
    score<- getScores(PcaCov)
    
    singlevalue<- sqrt(getEigenvalues(PcaCov))
    eigenvalues = singlevalue^2
    n <- NROW(score)
    singlevalue <- singlevalue * sqrt(n)
  }
  else if (model == "hSREG"){
    rlm.x<- rlm(y~env, data=Data, contrasts=list(env="contr.sum"),maxit = 50)
    residuals.x<- matrix(residuals(rlm.x), nrow = Ngen, ncol = Nenv)
    
    modeloHubert<- PcaHubert(residuals.x, mad=FALSE)
    
    loading<- getLoadings(modeloHubert)
    score<- getScores(modeloHubert)
    
    singlevalue<- sqrt(getEigenvalues(modeloHubert))
    eigenvalues = singlevalue^2
    n <- NROW(score)
    singlevalue <- singlevalue * sqrt(n)
  }
  else if (model == "ppSREG"){
    rlm.x<- rlm(y~env, data=Data, contrasts=list(env="contr.sum"),maxit = 50)
    residuals.x<- matrix(residuals(rlm.x), nrow = Ngen, ncol = Nenv)
    
    modeloProj<- PcaProj(residuals.x)
    loading<- getLoadings(modeloProj)
    score<- getScores(modeloProj)
    
    singlevalue<- sqrt(getEigenvalues(modeloProj))
    eigenvalues = singlevalue^2
    n <- NROW(score)
    singlevalue <- singlevalue * sqrt(n)
  }
  
  coordgenotype <- score
  coordenviroment <- loading
  singlevalue_scale <- singlevalue
  
  
  if(SVP=="row"){
    scale=0
  }
  if(SVP=="column"){
    scale=1
  }
  if(SVP=="symmetrical"){
    scale=0.5
  }
  
  if(scale < 0 || scale > 1) warning("'scale' is outside [0, 1]")
  if(scale != 0) singlevalue_scale <- singlevalue_scale^scale else singlevalue_scale <- 1
  
  coordgenotype <- t(t(score) / singlevalue_scale)
  coordenviroment <- t(t(loading) * singlevalue_scale)
  
  vartotal = round(as.numeric(sum(eigenvalues)),2)
  varexpl = round(as.numeric((eigenvalues/vartotal) *100), 2)
  
  centering="tester"
  scaling = "none"
  labelaxes <- paste("Component ",1:ncol(diag(singlevalue)), sep = "")
  
  rSREGModel=list(model=model,
                coordgenotype=coordgenotype,
                coordenviroment=coordenviroment,
                eigenvalues=eigenvalues,
                vartotal=vartotal,
                varexpl=varexpl,
                labelgen=labelgen,
                labelenv=labelenv,
                labelaxes=labelaxes,
                Data=residuals.x,
                SVP=SVP)
  class(rSREGModel)<-"rSREGModel"
  return(rSREGModel)
}

test_that("Validación de entradas y errores iniciales", {
  # Error por falta de Data
  expect_error(rSREGModel(genotype="gen"), "Need to provide Data data frame")
  
  # Error por presencia de NAs
  dat_na <- yan.winterwheat
  dat_na[1, "yield"] <- NA
  expect_error(rSREGModel(dat_na, "gen", "env", "yield"), "Missing data in input data frame")
  
  # Error por argumentos de clase incorrecta
  expect_error(rSREGModel(yan.winterwheat, genotype = 123, environment="env", response="yield"), "genotype")
  expect_error(rSREGModel(yan.winterwheat, "gen", "env", "yield", model = "Inexistente"), "model")
})

test_that("Estructura y clase del objeto de salida", {
  model_sreg <- rSREGModel(yan.winterwheat, "gen", "env", "yield")
  expect_s3_class(model_sreg, "rSREGModel")
  expect_named(model_sreg, c("model", "coordgenotype", "coordenviroment", 
                             "eigenvalues", "vartotal", "varexpl", "labelgen", 
                             "labelenv", "labelaxes", "Data", "SVP"))
})

test_that("Partición de Valores Singulares (SVP) y Varianza", {
  m_sym <- rSREGModel(yan.winterwheat, "gen", "env", "yield", SVP = "symmetrical")
  m_row <- rSREGModel(yan.winterwheat, "gen", "env", "yield", SVP = "row")
  
  expect_equal(m_sym$eigenvalues, m_row$eigenvalues)
  expect_false(identical(m_sym$coordgenotype, m_row$coordgenotype))
})

#'Weighted GabrielEigen imputation method
#'
#'@param DBmiss a data frame or matrix that contains the genotypes in the rows
#'  and the environments in the columns when there are no replications of the
#'  experiment.
#'@param Winf inferior weight
#'@param Wsup superior weight
#'
#'@references Arciniegas-Alarcón S., García-Peña M., Krzanowski W.J., Dias
#'  C.T.S. (2014). An alternative methodology for imputing missing data in
#'  trials with genotype-by-environment interaction: some new aspects.
#'  Biometrical Letters 51, 75-88.
#'
#'@return A list containing:
#'  \itemize{
#'    \item Peso: weight that provides the best predictive difference;
#'    \item NumeroIterWGabriel: the final number of iterations;
#'    \item CritConvergWGabriel: the maximum change of the estimated
#'    values for missing cells in the last step of iteration (the precision of
#'    convergence);
#'    \item GabrielWImput: the imputed matrix (filled in with the
#'    missing values estimated by the Weighted GabrielEigen procedure).
#'  }
#'
#'@keywords internal
#'@importFrom MASS ginv
#'

WGabriel<-function(DBmiss,Winf,Wsup){

  if (missing(DBmiss)) stop("Need to provide DBmiss data frame or matrix")
  stopifnot(
    class(DBmiss) %in% c("matrix", "data.frame"),
    class(Winf) == "numeric",
    class(Wsup) == "numeric"
  )


  ######
  ######Funci?n Gabriel Pesado
  ######
  Gabriel.Pesado<-function(X,w){
    nfilasX<-nrow(X)
    ncolX<-ncol(X)
    #nfilasX
    #ncolX
    if(ncolX>nfilasX) {X<-t(X)}
    
    XMISSING<-X
    is.matrix(X) 
    totalelementos<-nfilasX*ncolX

    indicamissing<-is.na(XMISSING)
    indicadora<-indicamissing*1
    totalfaltantes<-sum(indicadora)
    medcol<-t(matrix(colMeans(XMISSING, na.rm=TRUE))) 


    posicfaltantes<-matrix(0,totalfaltantes,2)
    indi3<-0
    for (indi1 in 1:nfilasX){
      for(indi2 in 1:ncolX){
        if (indicadora[indi1,indi2]==1){
          indi3<-indi3+1
          posicfaltantes[indi3,1]=indi1
          posicfaltantes[indi3,2]=indi2
          XMISSING[indi1,indi2]=medcol[1,indi2]
        } 
      }
    }


    completed_matrix<-XMISSING
    maxiter<-1000
    iter<-0
    desvest<-sqrt(t(diag(var(XMISSING))))
    #print(desvest)
    Xmissingestand<-scale(XMISSING)
    Prediction_Gabriel<-matrix(0,nfilasX,ncolX)
    epsilon<-1*(10**(-6))
    stabilitycrit<-1
    stabilityiter<-1



    while (stabilitycrit>epsilon & stabilityiter<=30){


      for (i1 in 1:totalfaltantes){

        posic_linha<-posicfaltantes[i1,1]
        posic_col<-posicfaltantes[i1,2]
        auxiliar1<-rbind(Xmissingestand[posic_linha,],Xmissingestand[-posic_linha,])
        auxiliar2<-cbind(auxiliar1[,posic_col],auxiliar1[,-posic_col])
        #print(i1)
        #print(auxiliar2)
        Y1puntoTRANS<-auxiliar2[1,2:ncolX]
        Ypunto1<-auxiliar2[2:nfilasX,1]
        Ybarra11<-auxiliar2[-1,-1]

        Ybarra11_Estand<-scale(Ybarra11)
        dvs_Ybarra11_Estand<-svd(Ybarra11_Estand)
        autovalores<-matrix(dvs_Ybarra11_Estand$d)**2
        sum_autovalores<-sum(autovalores)
        CritCalinski1<-(autovalores/sum_autovalores)
        CritCalinski2<-matrix(cumsum(CritCalinski1))
        CritCalinski3<-matrix(1,ncol(Ybarra11_Estand),1)
        for (Calins in 1:ncol(Ybarra11_Estand)){
          if (CritCalinski2[Calins,1]>0.70){CritCalinski3[Calins,1]=0}
        } # FIN de for (Calins in 1:ncol(Ybarra11_Estand)){
        f<-sum(CritCalinski3)+1
        #print(i1)
        #print(f)


        #print(Y1puntoTRANS)
        #print(Ypunto1)
        #print(Ybarra11)
        dvs.Ybarra11<-svd(Ybarra11)
        u_Ybarra11<-dvs.Ybarra11$u
        v_Ybarra11<-dvs.Ybarra11$v
        d_Ybarra11<-diag(dvs.Ybarra11$d)

        pred_estand<-w*Y1puntoTRANS%*%v_Ybarra11[,1:f]%*%MASS::ginv(d_Ybarra11[1:f,1:f])%*%t(u_Ybarra11[,1:f])%*%Ypunto1
        completed_matrix[posic_linha,posic_col]<-medcol[1,posic_col]+(pred_estand*desvest[1,posic_col])



      } 

      stabilitycrit<-(sum((completed_matrix-XMISSING)**2))/(sum((XMISSING*indicadora)**2))
      stabilityiter<-stabilityiter+1
      XMISSING<-completed_matrix
      Xmissingestand<-scale(completed_matrix)
      medcol<-t(matrix(colMeans(completed_matrix)))
      desvest<-sqrt(t(diag(var(completed_matrix))))

      #print(stabilityiter)
      #print(stabilitycrit)
      ######print (f)
      #print(completed_matrix)



    }
    list(Peso=w,NumeroIterGabriel=stabilityiter,CritConvergGabriel=stabilitycrit,GabrielImput=completed_matrix)
  }




  nfilasDBmiss<-nrow(DBmiss)
  ncolDBmiss<-ncol(DBmiss)
  indicaDBmissing<-is.na(DBmiss)
  indicaDBcomplete<-1-indicaDBmissing
  posicobservadosDBmiss<-which(indicaDBmissing != 1, TRUE) 
  posicfaltantesDBmiss<-which(indicaDBmissing != 0, TRUE)  
  TotalFaltantesDBmiss<-nrow(posicfaltantesDBmiss)
  TotalObservadosDBmiss<-nrow(posicobservadosDBmiss)
  MatrizPesos<-matrix(seq(Winf,Wsup,0.1))
  NumTotdePesos<-nrow(MatrizPesos)
  VetorResultDBmiss1<-matrix(0,1,2)

  for (w in 1:NumTotdePesos){
    EstimaObsDBmiss<-matrix(0,nfilasDBmiss,ncolDBmiss)
    for (iobsDB in 1:TotalObservadosDBmiss){
      CVDB<-DBmiss
      CVDB[posicobservadosDBmiss[iobsDB,1],posicobservadosDBmiss[iobsDB,2]]<-NA
      CVDB.Imputa<-Gabriel.Pesado(CVDB,MatrizPesos[w,1])
      EstimaObsDBmiss[posicobservadosDBmiss[iobsDB,1],posicobservadosDBmiss[iobsDB,2]]<-CVDB.Imputa$GabrielImput[posicobservadosDBmiss[iobsDB,1],posicobservadosDBmiss[iobsDB,2]]
      #print(iobsDB)
      #tempo1<-sum(is.na(CVDB))
      #print (tempo1)
      #print(CVDB)
      #print(CVKR.Imp)
      #print(MatrizValidacion)
    } 
    DBcomplete<-DBmiss
    DBcomplete[is.na(DBcomplete)] <- 0
    RMSPD.Obs<-sqrt((sum((EstimaObsDBmiss-(DBcomplete*indicaDBcomplete))^2))/TotalObservadosDBmiss)

    VetorResultDBmiss<-matrix(0,1,2)
    VetorResultDBmiss[1,1]<-MatrizPesos[w,1]
    VetorResultDBmiss[1,2]<-RMSPD.Obs
    VetorResultDBmiss1<-rbind(VetorResultDBmiss1,VetorResultDBmiss)

    #print(paste("Num.PesoExt",w), sep=" ")
    print(paste("  PesoExterno = ",MatrizPesos[w,1], sep= " "))

  }

  ResultadosPesosDB<-VetorResultDBmiss1[-1,]
  colnames(ResultadosPesosDB)<-c("PesoW","RMSPD.Obs")
  MinimoRMSPD<-min(ResultadosPesosDB[,2])
  FilaconMinimoRMSPD<-subset(ResultadosPesosDB,ResultadosPesosDB[,2]==MinimoRMSPD)
  Wsugerido<-FilaconMinimoRMSPD[1,1]


  Liminf2<-Wsugerido-0.1
  Limsup2<-Wsugerido+0.1
  MatrizPesos2<-matrix(seq(Liminf2,Limsup2,0.01))
  NumTotdePesos2<-nrow(MatrizPesos2)
  VetorResultDBmiss22<-matrix(0,1,2)

  for (w2 in 1:NumTotdePesos2){
    EstimaObsDBmiss2<-matrix(0,nfilasDBmiss,ncolDBmiss)
    for (iobsDB2 in 1:TotalObservadosDBmiss){
      CVDB2<-DBmiss
      CVDB2[posicobservadosDBmiss[iobsDB2,1],posicobservadosDBmiss[iobsDB2,2]]<-NA
      CVDB.Imputa2<-Gabriel.Pesado(CVDB2,MatrizPesos2[w2,1])
      EstimaObsDBmiss2[posicobservadosDBmiss[iobsDB2,1],posicobservadosDBmiss[iobsDB2,2]]<-CVDB.Imputa2$GabrielImput[posicobservadosDBmiss[iobsDB2,1],posicobservadosDBmiss[iobsDB2,2]]
      #print(iobsDB)
      #tempo1<-sum(is.na(CVDB))
      #print (tempo1)
      #print(CVDB)
      #print(CVKR.Imp)
      #print(MatrizValidacion)
    } 
    DBcomplete2<-DBmiss
    DBcomplete2[is.na(DBcomplete2)] <- 0
    RMSPD.Obs2<-sqrt((sum((EstimaObsDBmiss2-(DBcomplete2*indicaDBcomplete))^2))/TotalObservadosDBmiss)

    VetorResultDBmiss2<-matrix(0,1,2)
    VetorResultDBmiss2[1,1]<-MatrizPesos2[w2,1]
    VetorResultDBmiss2[1,2]<-RMSPD.Obs2
    VetorResultDBmiss22<-rbind(VetorResultDBmiss22,VetorResultDBmiss2)

    #print(w2)
    #print(MatrizPesos2[w2,1])
    print(paste("    PesoInterno = ",MatrizPesos2[w2,1], sep= " "))



  } 

  ResultadosPesosDB2<-VetorResultDBmiss22[-1,]
  colnames(ResultadosPesosDB2)<-c("PesoW","RMSPD.Obs")
  MinimoRMSPD2<-min(ResultadosPesosDB2[,2])
  FilaconMinimoRMSPD2<-subset(ResultadosPesosDB2,ResultadosPesosDB2[,2]==MinimoRMSPD2)
  Wsugerido2<-FilaconMinimoRMSPD2[1,1]
  WGabriel2<-Gabriel.Pesado(DBmiss,Wsugerido2)

  list(Peso=WGabriel2$Peso,NumeroIterWGabriel=WGabriel2$NumeroIterGabriel,CritConvergWGabriel=WGabriel2$CritConvergGabriel,GabrielWImput=WGabriel2$GabrielImput)

}

---
title: "How to use the geneticae package"
subtitle: "Statistical Tools for the Analysis of Multi-Environment Agronomic Trials"
author: |
    | Julia Angelini
    | Marcos Prunello
    | Gerardo Cervigni
    |
    | Centro de Estudios Fotosintéticos y Bioquímicos
    | Universidad Nacional de Rosario
    | Rosario, Argentina
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{How to use the geneticae package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(warning = F, message = F, out.width = "60%")
library(dplyr)
```

<style type="text/css">

body{
text-align: justify}
</style>
  
  
<!-- devtools::build_vignettes() -->
# Getting Started

__Installing the package.__ To install the released version of `geneticae` from [CRAN](https://CRAN.R-project.org):

```{r, eval=F}
install.packages("geneticae")
```

You can install the development version from our [GitHub repo](https://github.com/jangelini/geneticae) with:

``` {r, eval=F}
# install.packages("devtools")
devtools::install_github("jangelini/geneticae")
```



__Loading the package.__ Once the `geneticae` package is installed, it needs to be loaded by:

```{r}
library(geneticae)
```

__Help files.__  Detailed information on `geneticae` package functions can be obtained from  help files using `help(package="geneticae")`.
The help file for a function, for example `imputation` can be obtained using `?imputation` or `help(imputation)`.


# Introduction

Understanding the relationship between crops performance and environment is a
key problem for plant breeders and geneticists. In advanced stages of breeding
programs, in which few genotypes are evaluated, multi-environment trials (MET)
are one of the most used experiments. Such studies test a number of genotypes in
multiple environments in order to identify the superior genotypes according to
their performance. In these experiments, crop performance is modeled as a
function of genotype (G), environment (E) and genotype-environment interaction
(GEI). The presence of GEI generates differential genotypic responses in the
different environments (Angelini et al., 2019; Crossa, 1990; Kang and Magari,
1996). Therefore appropriate statistical methods should be used to obtain an
adequate GEI analysis, which is essential for plant breeders (Giauffret et al.,
2000).

The average performance of genotypes through different environments can only be
considered in the absence of GEI (Yan and Kang, 2003). However, GEI is almost
always present and the comparison of the mean performance between genotypes is
not enough. The most widely used methods to analyze MET data are based on
regression models, analysis of variance (ANOVA) and multivariate techniques. In
particular, two statistical models are widely used among plant breeders as they
provide useful graphical tools for the study of GEI: the Additive Main effects
and Multiplicative Interaction model (AMMI) (Kempton, 1984; Gauch, 1988) and the
Site Regression Model (SREG) (Cornelius et al., 1996; Gauch and Zobel, 1997).
However, these models are not always efficient enough to analyze MET data
structure of plant breeding programs. They present serious limitations in the
presence of atypical observations and missing values, which occur very
frequently. To overcome this, several imputation alternatives, a robust AMMI 
(Rodrigues et al., 2016) and SREG alternative (Angelini et al., 2022) were recently
proposed in literature.

The `geneticae` package was created to gather in one place the most useful
functions for this type of analysis and it also implements new methodology which
can be found in recent literature. More importantly, `geneticae` is the first
package to implement the robust AMMI models proposed by Rodrigues et al. (2016),
the robusts SREG proposed by Angelini et al. (2022) and new imputation methods
proposed by Angelini et al. (2024). In addition, there is no need to preprocess
the data to use the `geneticae` package, as it the case of some previous
packages which require a data frame or matrix containing genotype by environment
means  with  the genotypes in rows and the environments in columns. In this
package, data in long format is required. There is no restriction on columns
names of genotypes, environments, repetitions (if any) and phenotypic traits of
interest. Also, extra information that will not be used in the analysis may be
present in the dataset. Finally, `geneticae` offers a wide variety of options to
customize the biplots, which are part of the graphical output of these methods.


# Datasets

The `geneticae` package utilizes two datasets to illustrate the methodology
included to analyse MET data.

* `yan.winterwheat` dataset: yield of 18 winter wheat varieties grown in nine
environments in Ontario at 1993. Although four blocks or replicas in each
environment were performed in the experiment, only yield mean for each variety
and environment combination was available in the dataset obtained from the
[agridat](https://CRAN.R-project.org/package=agridat) package (Wright, 2020).


```{r}
library(agridat)
data(yan.winterwheat)
head(yan.winterwheat)
```

* `plrv` dataset: resistance study to PLRV (Patato Leaf Roll Virus) causing leaf
curl. 28 genotypes were experimented at 6 locations in Peru. Each clone was
evaluated three times in each environment, and yield, plant weight and plot were
registered. This dataset is available in `geneticae` and was obtained from the
[agricolae](https://CRAN.R-project.org/package=agricolae) package (de Mendiburu,
2020).


```{r}
data(plrv)
head(plrv)
```


# Statistical models for multi-environment trials

## AMMI model

The *AMMI model* (Gauch, 1988) is widely used to analyse the effect of GEI.
This model includes two stages. First, an ANOVA is performed to obtain estimates
for the additive main effects of environments and genotypes. Secondly, the
residuals from the ANOVA are arranged in a matrix with genotypes in the rows and
environments in the columns and a singular value decomposition (SVD) is applied
in order to explore patterns related to GEI, still present in the residuals. The
result of the first two multiplicative terms of the SVD is often presented in a
biplot called GE and represents a two-rank approximation of GEI effects.

The `rAMMI()` function returns the GE biplot. Data in long format is required by
this function, i.e. each row corresponds to one observation and each column to
one variable (genotype, environment, repetition (if any) and the observed
phenotype). If each genotype has been evaluated more than once at each
environment, the phenotypic mean for each combination of genotype and
environment is internally calculated and then the model is estimated. Extra
variables that will not be used in the analysis may be present in the dataset.
Missing values are not allowed (but can be imputated, see below).

The GE biplot for `yan.winterwheat` dataset is shown in Figure 1 along with the
sentence used to obtain it. The first argument is the input dataset, then the
names of the columns in which the necessary information to apply the technique
is found and also the model to be obtained are indicated.  Optionally, the
percentage of GEI explained by the GE biplot can be added as a footnote with
`footnote = T`, as well as a tittle with `titles = T`. In this example, BH93,
KE93 and OA93 are the environments that contribute the most to the interaction
as their vectors are the longest ones. The cultivars _m12_ and _Kat_ present
similar interaction patterns (their markers are close to each other in the biplot)
and they are very different from _Ann_ and _Aug_, for example. 
The closeness between the cultivar _Dia_  and the environment BH93 indicates a 
strong positive association between them, which means that BH93 is a 
extremely favorable environment for that genotype. As OA93 and _Luc_ markers are
opposite, this environment is considerably unfavorable for that genotype.
Finally, _Cas_ and _Reb_ are close to the origin, which means that they adapt
equally to all environments.

```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 1: GE biplot based on yield data of 1993 Ontario winter wheat performance trials.  The 71.66% of GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'}

rAMMI_clasic <- rAMMIModel(yan.winterwheat, genotype = "gen", environment = "env", 
      response = "yield", type = "AMMI")

rAMMIPlot(rAMMI_clasic, titles = TRUE, footnote = TRUE)

```

The AMMI model, in its standard form, assumes that no outliers are present in
the data. To overcome the problem of data contamination with outlying
observations, Rodrigues et al. (2016) proposed five robust AMMI models, which
can be obtained in two stages: (i) fitting a robust regression model with an
M-Huber estimator (Huber, 1981) to replace the ANOVA model; and (ii) using a
robust SVD or principal components analysis (PCA) procedure to replace the
standard SVD. Until now, robust AMMI models  were not available in any R
package. All robust biplots proposed by Rodrigues et al. (2016) can be obtained
using `rAMMI()`. The argument `type` can be used to specify the type of
model to be fitted (`"rAMMI"`, `"hAMMI"`, `"gAMMI"`, `"lAMMI"` or `"ppAMMI"`).0
Since the sample `yan.winterwheat` dataset does not present outliers, the
conclusions obtained with robust biplots will not differ from those made with
the classic biplot (Rodrigues et al., 2016). Thus, no interpretation of the
robust biplots is presented in this tutorial.


## Site Regression model

The *Site Regression model* (SREG, also called *genotype plus
genotype-by-environment model* or *GGE model*) is another powerful tool for the
analysis and interpretation of MET data in breeding programs. In this case, an
ANOVA is performed to obtain estimates for the additive main effects of
environments and a SVD is performed on the residuals matrix in order to explore
patterns related to genotype (G) and GEI. However, ANOVA and SVD are sensitive to
atypical observations, which are common in MET. To overcome
this problem, three robust models were proposed by Angelini et al. (2022) to obtain valid
results even in the presence of outliers. The current package implements these 
methods, allowing users to perform robust SREG analysis easily. As `yan.winterwheat` 
dataset does not present outliers, the conclusions obtained with robust biplots 
will not differ from those made with the classic biplot (Angelini et al., 2022). Thus,
no interpretation of the robust biplots is presented in this tutorial.

As `rAMMI()` function, `GGEmodel()` data needs to be presented in a long format
and repetitions or extra variables in the dataset are allowed. All the
combinations between genotypes and environments must be present.

```{r}
GGE1 <- rSREGModel(yan.winterwheat, genotype = "gen", environment = "env", 
                 response = "yield")
```

The output from `GGEmodel()` is a list with the following elements:

* `model`: method for fitting the SREG model: `"SREG"`,`"CovSREG"`,`"hSREG"`or `"ppSREG"`.
* `coordgenotype`: plot coordinates for all genotypes in each component. 
* `coordenviroment`: plot coordinates for all environments in each component.
* `eigenvalues`: vector of eigenvalues for each component.
* `vartotal`: overall variance.
* `varexpl`: percentage of variance explained by each component. 
* `labelgen`: genotype names.
* `labelenv`: environment names.
* `axes`: axis labels.
* `Data`: centered input data.
* `SVP`: SVP method.

The result of the first two multiplicative terms of the SVD is often presented
in a GGE biplot (Yan et al., 2000), which represents a rank-two approximation of
the G + GEI effects. Plant breeders have found GGE biplots as an useful tools
for the analysis of mega-environment (Yan et al., 2001; Yan and Rajcan, 2002)
and genotype and environment evaluation (Bhan et al., 2005; Kang et al., 2006;
Yan et al., 2007). The GGE biplot addresses many issues relative to genotype and
test environment evaluation. Considering the average performance of each
genotype, this plot can be used to evaluate specific and general adaptation. In
addition, environments can be visually grouped according to their ability to
discriminate among genotypes and their representativeness of other test
environments. GGE biplot reveals the *which-won-where* pattern and allows to
recommend specific genotypes for each environment (Yan and Tinker, 2005).

Using the output from `GGEmodel()`, `GGEPlot()` builds several GGE biplots
views, in which cultivars are shown in lowercase and environments in uppercase.
The plot also displays the methods used for centering, scaling and SVD.
Optionally, the percentage of G + GEI explained by the two axes can be added as
a footnote with `footnote = T`, as well as a tittle with `titles = T`.

A basic biplot is produced with the option `type="Biplot"` (Figure 2). In this
example the 78% of G + GE variability is explained by the fist two
multiplicative terms. The angles between genotypes markers and environments
vectors are considered to understand this plot. Thus, for example, _Kat_
performs below the average in all environments, as it has an angle greater than
90$°$ with all environments. On the other hand, _Fun_ presents an above-average
performance in all locations except OA93 and KE93, as indicated by the acute angles.
The length of the environment vectors is a measure of the environment's ability to
discriminate between crops.

```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 2: GGE biplot based on yield data of 1993 Ontario winter wheat performance trials. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'}
rSREGPlot(GGE1, type = "Biplot", footnote = F, titles = F)
```

Breeders usually want to identify the most suitable cultivars for a particular
environment of interest, i.e., OA93. To do this with GGE biplots, Yan and Kang
(2003) suggest drawing a line that passes through the environment marker and the
biplot origin, which may be referred to as the OA93 axis. The performance of the
cultivars in this particular environment can be ranked projecting them onto this
axis.
This can be done by setting `type = "Selected Environment"` and providing 
the name of the environment (OA93) in
`selectedE` (Figure 3). Thus, at OA93, the highest-yielding cultivar was _Zav_,
and the lowest-yielding cultivar was _Luc_. The line that passes through the
biplot origin and is perpendicular to the OA93 axis separates genotypes that
yielded above and below the mean in this environment.

```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 3: comparison of cultivar performance in a selected environment (OA93). The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms.'}
rSREGPlot(GGE1, type = "Selected Environment", selectedE = "OA93", 
        footnote = F, titles = F)
```

Another goal of plant breeders is to determine which is the most suitable
environment for a genotype. Yan and Kang (2003) suggest plotting a line that
passes through the origin and a cultivar marker, i.e., _Kat_. To obtain this GGE
biplots view the argument  `type = "Selected Genotype"` and `selectedG = "Kat"`
must be indicated (Figure 4). Environments are classified along the genotype
axis in the direction indicated by the arrow. The perpendicular axis separates
the environments in which the cultivar presented a performance below or above
the average. In this example, _Kat_ presented a performance below the average in
all the environments studied.

```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 4: comparison of the performance of cultivar Luc in different environments. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. '}
rSREGPlot(GGE1, type = "Selected Genotype", selectedG = "Kat", 
        footnote = F, titles = F)
```

It is also possible to compare two cultivars, i.e. _Kat_ and _Cas_, linking them
with a line and a segment perpendicular to it. To obtain this GGE biplots view
the argument  `type = "Comparison of Genotype"` and the genotypes to be compared
`selectedG1 = "Kat"` and `selectedG2 = "Cas"` must be indicated (Figure 5).
_Cas_ was more yielding than _Kat_ in all environments as they all are in the 
same side of the perpendicular line as _Cas_.

```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 5: comparison of the cultivars _Kat_ and _Cas_. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'}
rSREGPlot(GGE1, type = "Comparison of Genotype", 
        selectedG1 = "Kat", selectedG2 = "Cas", 
        footnote = F, titles = F, axis_expand = 1.5)
```

The polygonal view of the GGE biplots provides an effective way to visualize the
*which-won-where* pattern of MET data (Figure 6). Cultivars in the vertices of
the polygon (_Fun_,_Zav_, _Ena_, _Kat_ and _Luc_) are those with the longest
vectors, in their respective directions, which is a measure of the ability to
respond to environments. The vertex cultivars are, therefore, among the most
responsive cultivars; all other cultivars are less responsive in their
respective directions. 

The dotted lines are perpendicular to the polygon sides and divide the biplot
into mega-environments, each of which has a vertex cultivar, which is the one
with the highest yield (phenotype) in all environments found in it. OA93 and
KE93 are in the same sector, separated from the rest of the biplot by two
perpendicular lines, and _Zav_ is the highest-yielding cultivar in this sector.
_Fun_ is the highest-yielding cultivar in its sector, which contains seven
environments, namely, EA93, BH93, HW93, ID93, WP93, NN93, and RN93. No
environments fell in the sectors with _Ena_, _Kat_, and _Luc_ as vertex
cultivars. This indicates that these vertex cultivars were not the best in any
of the test environments. Moreover, these cultivars were the poorest in some or
all of the environments.

```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 6: polygon view of the GGE biplot, showing which cultivars presented highest yield in each environment. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'}
rSREGPlot(GGE1, type = "Which Won Where/What", footnote = F,
        titles = F, axis_expand = 1.5)

```

Selecting cultivars within each mega-environments is an issue among plant
breeders. Figure 6 clearly suggests that _Zav_ is the best cultivar for OA93
and KE93, and _Fun_ is the best cultivar for the other locations. However,
breeders do not select a single cultivar in each megaenvironment. Instead, they
evaluate all cultivars in order to get an idea of their performance (yield and
stability). 

In the GGE biplot it is also possible to visualize mean yield and stability of
genotypes in yield units _per se_ (Figure 7 and 8). The GGE biplot based on
*genotype-focused scaling*, obtained indicating `SVP = "row"` in `GGEmodel()`,
provides an useful way to visualize both mean performance and stability of
the tested genotypes. This is because the unit of both axes for the genotypes is
the original unit of the data.

Visualization of the mean and stability of genotypes is achieved by drawing an
average environment coordinate (AEC). For example, Figure 7 shows the AEC for
the mega-environment composed of he environments BH93, EA93, HW93, ID93, NN93,
RN93, WP93. The abscissa represents the G effect, thus, the cultivars are ranked
along the AEC abscissa. Cultivar _Fun_ was clearly the highest-yielding
cultivar, on average, in this mega-environment, followed by _Cas_ and _Har_,and
_Kat_ was the poorest. The AEC ordinate approximate the GEI associated with each
genotype, which is a measure of the variability or instability of the genotype.
_Rub_ and _Dia_ are more variable and less stable than other cultivars, by the
contrary, _Cas_, _Zav_, _Reb_, _Del_, _Ari_, and _Kar_, were more stable.


```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 7: average environment view of the GGE biplot based on genotype-focused scaling, showing mean yield and stability of genotypes. '}
data <- yan.winterwheat[yan.winterwheat$env %in% c("BH93", "EA93","HW93", "ID93",
                                                   "NN93", "RN93", "WP93"), ]
data <- droplevels(data)
GGE2 <- rSREGModel(data, genotype = "gen", environment = "env", 
                 response = "yield", SVP = "row")

rSREGPlot(GGE2, type = "Mean vs. Stability", footnote = F, titles = F, sizeEnv = 0)

```


Figure 8 compares the cultivars to the "ideal" one with the highest
yield and absolute stability. This ideal cultivar is represented by a small
circle and is used as a reference, as it rarely exists. The distance between
cultivars and the ideal one can be used as a measure of convenience. Concentric
circles help to visualize these distances. In the example, _Fun_ is the closest
one to the ideal crop, and therefore the most desirable one, followed by _Cas_
and _Hay_, which in turn are followed by _Rum_, _Ham_, _Rub_, _Zav_, _Del_ and
_Reb_, etc.


```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 8: Classification of genotypes with respect to the ideal genotype. Genotype-focused scaling is used.', warning=FALSE}
rSREGPlot(GGE2, type = "Ranking Genotypes", footnote = F, titles = F, sizeEnv = 0)
```

Although METs are performed to study cultivars, they are equally useful for the
analysis of the environments. This includes several aspects: (i) evaluating
whether the target region belongs to one or more megaenvironments; (ii)
identifying better test environments; (iii) detecting redundant environments
that do not provide additional information on cultivars; and (iv) determining
environments that can be used for indirect selection. To obtain GGE biplots for
comparing environments the *environment-focused scaling* should be used as is
most informative of interrelationships among them (Figure 9 and 10). This is
obtained indicating `SVP = "column"` in `GGEmodel()`.

In Figure 9 environments are connected to the origin through vectors, allowing
us to understand the interrelationships between them. The coefficient of correlation
between two environments it is approximated by the cosine of the angle formed by
the respective vectors. In this example the relation between the environments
for the mega-environment with BH93, EA93, HW93, ID93, NN93, RN93 and WP93 is
considered. The angle between the vectors for the environments NN93 and WP93 is
approximately 10$º$; therefore, they are closely related; while RN93 and BH93
present a weak negative correlation since the angle is slightly greater than
90$º$. The cosine of the angles does not translate precisely into coefficients
of correlation, since the biplot does not explain all the variability in the
dataset. However, they are informative enough to understand the
interrelationship between test environments.

```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 9: Relationship between environments. Environment-focused scaling is used.'}

GGE3 <- rSREGModel(data, genotype = "gen", environment = "env", 
                 response = "yield", SVP = "column")
rSREGPlot(GGE3, type = "Relationship Among Environments", footnote = F, titles = F)
```


Discrimination ability as well as representativeness with respect to the target
environment are fundamental measures for an environment. An ideal test
environment should be both discriminating and representative. If it does not
have the ability to discriminate, it does not provide information on cultivars
and is therefore of no use. At the same time, if it is not representative, not
only does it lack usefulness but it can also provide biased information on the
evaluated cultivars.

To visualize these measurements, an average environment coordinate is defined
and the center of a set of concentric circles represents the ideal environment.
Figure 10 shows the GGE biplots view for the mega-environment with BH93, EA93,
HW93, ID93, NN93, RN93 and WP93. The angle between the vector of an environment
and the AEC provides a measure of representativeness. Therefore, EA93 and ID93
are the most representative, while RN93 and BH93 are the least representative of
the average environment, when the mega-environment is analyzed. On the other
hand, an environment to be discriminative  must be close to the ideal
environment. HW93 is the closest to ideal environment and therefore the most
desirable of the mega-environment, followed by EA93 and ID93. By the contrary,
RN93 and BH93 were the least desirable test environments of this
mega-environment.


```{r, fig.width=6.5, fig.height=6, fig.align='center', dpi=300, out.width="60%", fig.cap='Figure 10: classification of environments with respect to the ideal environment. Environment-focused scaling is used.'}
rSREGPlot(GGE3, type = "Ranking Environments", footnote = F, titles = F)
```


## Robust Site Regression model
While the previous example used a dataset without contamination, MET data often
contain atypical observations due to environmental stress, pests, or measurement
errors. To illustrate the robust capabilities of the package, we refer to the
study by Angelini et al. (2022) using the `lavoranti.eucalyptus` dataset,
available in the agridat package (Wright 2020). In this case, a
distance-distance plot identified five atypical genotypes (1, 2, 21, 22, and
24).
As noted by Angelini et al. (2022), when outliers are present, the first PC
eigenvalues in the classic SREG model are often overestimated. This occurs
because standard SREG uses measures of variability that are highly influenced by
extreme values; consequently, outliers in the direction of the first principal
axes inflate the corresponding variances and artificially increase their
proportion of explained variability. In this context, the high percentage of
variability explained by a classic biplot can be spurious, reflecting the
presence of outliers rather than the actual main structure of the data.
The package allows fitting robust alternatives to overcome this inflation by
simply changing the method argument. The following code compares the classic
SREG with the robust approaches:


```{r, fig.width=12, fig.height=10, fig.align='center', dpi=300, out.width="100%", fig.cap='Figure 11: GGE biplot based on the height of 25 progenies of Eucalyptus grandis from trials carried out in seven environments in the southern and southeastern regions of Brazil. The scaling method used is symmetrical singular value partitioning. SREG (A), hSREG (B), CovSREG (C), and ppSREG (D).'}

library(patchwork)
library(ggplot2)
data("lavoranti.eucalyptus")

# Bellthorpe
lavoranti_Bellthorpe <- droplevels(subset(lavoranti.eucalyptus, origin=="Bellthorpe"))

Nenv <-length(levels(lavoranti_Bellthorpe$loc))
Ngen <- length(levels(lavoranti_Bellthorpe$gen))

lavoranti_Bellthorpe$genot <- rep(1:Ngen, Nenv)
lavoranti_Bellthorpe$genot <- as.factor(lavoranti_Bellthorpe$genot)


# SREG Classic
SREG_classic <- rSREGModel(lavoranti_Bellthorpe, genotype = "genot", 
                          environment = "loc", response = "height", 
                          model = "SREG")

# hSREG
hSREG <- rSREGModel(lavoranti_Bellthorpe, genotype = "genot", 
                    environment = "loc", response = "height", 
                    model = "hSREG")

# CovSREG
CovSREG <- rSREGModel(lavoranti_Bellthorpe, genotype = "genot", 
                    environment = "loc", response = "height", 
                    model = "hSREG")



# ppSREG
ppSREG <- rSREGModel(lavoranti_Bellthorpe, genotype = "genot", 
                    environment = "loc", response = "height", 
                    model = "ppSREG")

# Graphs
p1 <- rSREGPlot(SREG_classic, footnote = F, titles = T)
p2 <- hSREG_plot  <- rSREGPlot(hSREG, footnote = F, titles = T)
p3 <- CovSREG_plot <- rSREGPlot(CovSREG, footnote = F, titles = T)
p4 <- ppSREG_plot  <- rSREGPlot(ppSREG, footnote = F, titles = T)

grafico_robusto_SREG <- (p1 + p2) / (p3 + p4) + 
  plot_annotation(tag_levels = 'A') & 
  theme(plot.tag = element_text(size = 18, face = "bold"))

grafico_robusto_SREG
```

The display of genotypes and environments shows a different behavior for
the classic SREG biplot. The standard SREG biplot shows an overlap of
genotype markers in the direction of the first PC due to the dominant effect
of observations 2, 21, 22 and 24, which difficult the interpretation and
analysis of MET data (Figure 11 A). For example, as the length of the environ­
mental vectors is a measure of the environment’s ability to discriminate
between cultivars, we can conclude that environment 2 was one of the
most discriminating when the classical biplot is analyzed. By the contrary,
it is near the biplot origin when the robust ones were interpreted, indicating
the lack of discriminating ability; that is, it does not provide information
about the cultivars, and therefore, the test environment is useless (Figure 11
B–D). This contradiction occurs because of the influence of genotypes 2, 21,
22 and 24 which have the poorest performance in that environment (while all
the genotypes rendering above 22.60 in that environment give 17.79, 19.77,
20.45 and 20.36, respectively). Outliers concentrated in a single environment
may be the result of a particular climatic event or some pests/diseases that
have influenced the expected performance.
When considering the robust alternatives, the impact of the outlying
observations is reduced leading to an easier interpretation, with genotype
markers being more dispersed. Therefore, the use of robust alternatives
allowed the inclusion of those genotypes in the analysis, reducing their
influence, and consequently not distorting the results.

# Imputation methods

One major limitation of the AMMI and SREG models is that they require a complete
two-way data table. Although METs are designed so that all genotypes are
evaluated in all environments, missing values are very common due to measurement
errors or destruction of plants by animals, floods or harvest problems. In
addition, genotypes might be incorporated or discarded during the study because
of their promising or poor performance. The `imputation()` function includes
several methods to overcome the problem of missing data, some of which  have
been recently published and were not available in any R package until now, such
as the recently proposed EM-SREG and EM-bSREG methods (Angelini et al., 2024).
To present an example, some observations from the complete `yan.winterwheat` are
deleted:

```{r}
# Generating missing data
yan.winterwheat[1,3] <- NA
yan.winterwheat[3,3] <- NA
yan.winterwheat[2,3] <- NA
```


To impute missing values using the methods proposed by Angelini et al. (2024) `"EM-SREG"` and `"EM-bSREG"`, use the following code:

```{r}
imputation(yan.winterwheat, genotype = "gen", environment = "env", 
           response = "yield", type = "EM-SREG")


imputation(yan.winterwheat, genotype = "gen", environment = "env", 
           response = "yield", type = "EM-bSREG")
```

The other methods available in `geneticae` are: `"EM-SREG"`, `"EM-bSREG"`, `"EM-SVD"`, `"Gabriel"`, `"WGabriel"` and `"EM-PCA"`.
 


# References

Angelini, J., Faviere, G. S., Bortolotto, E. B., Arroyo, L., Valentini, G. H.,
and Domingo Lucio Cervigni, G. 2019. Biplot pattern interaction analysis and
statistical test for crossover and non-crossover genotype-by-environment
interaction in peach. Scientia Horticulturae, 252, 298–309. \n

Angelini, J., Faviere, G. S., Bortolotto, E. B., Domingo Lucio Cervigni, G, and
Quaglino, M. B. 2022. Handling outliers in multi-environment trial data analysis: 
in the direction of robust SREG model. Journal of Crop Improvement. \n

Angelini, J., Domingo Lucio Cervigni, G, and Quaglino, M. B. 2024. New imputation 
methodologies for genotype‐by‐environment data: an extensive study of properties 
of estimators. Euphytica. \n

Bhan, M.K., Pal, S., Rao, B.L., Dhar, A.K., and Kang, M.S. 2005. GGE biplot
analysis of oil yield in lemongrass. Journal of New Seeds, 7, 127–139.  \n

Cornelius, P.L., J. Crossa, and M.S. Seyedsadr. 1996. Statistical tests and
estimates of multiplicative models for GE interaction, p. 199–234. In: M.S. Kang
and H.G. Gauch, Jr. (Eds.), Genotype-by-environment interaction, CRC Press, Boca
Raton, FL. \n

Crossa, J. 1990. Statistical Analyses of Multilocation Trials. Advances in
Agronomy, 55–85. \doi{10.1016/s0065-2113(08)60818-4} \n

Dumble, S. 2017. GGEBiplots: GGE Biplots with 'ggplot2'. R package version
0.1.1.  \n

de Mendiburu, F. 2020. agricolae: Statistical Procedures for Agricultural
Research. R package version 1.3-2. \n

Gauch, H.G., Jr. 1988. Model selection and validation for yield trials with
interaction, Biometrics, 44, 705–715. \n

Gauch H.G. and R.W. Zobel. 1997. Identifying mega-environments and targeting
genotypes. Crop Science, 37, 311–326.  \n

Giauffret, C., Lothrop, J., Dorvillez, D., Gouesnard, B., and Derieux, M., 2000.
Genotype x environment interactions in maize hybrids from temperate or highland
tropical origin. Crop Science, 40, 1004-1012. \n

Huber, P.J. 1981. Robust Statistics. Wiley, New York. \n

Kang, M.S., Aggarwal, V.D., and Chirwa, R.M. 2006. Adaptability and stability of
bean cultivars as determined via yield-stability statistic and GGE biplot
analysis. Journal of Crop Improvement, 15, 97–120. \n

Kang, M.S. and Magari, R. 1996. New developments in selecting for phenotypic
stability in crop breeding, p. 1–14. In: M.S. Kang and H.G. Gauch, Jr. (Eds.),
Genotype-by-environment interaction, CRC Press, Boca Raton, FL. \n

Kempton, R.A. 1984. The use of biplots in interpreting variety by environment
interactions. The Journal of Agricultural Science, 103, 123–135.  \n

Rodrigues,  P.C.,  Monteiro,  A., and Lourenço,  V.M. 2016.  A  robust  AMMI
model  for  the  analysis  of  genotype-by-environment  data. Bioinformatics,32,
58–66.  \n

Wright, K. 2020. agridat: Agricultural Datasets. R package version 1.17. \n

Yan, W., Cornelius, P.L., Crossa, J., and Hunt, L.A. 2001. Two types of GGE
biplots for analyzing multi-environment trial data. Crop Science, 41, 656–663. \n

Yan, W., Hunt, L.A., Sheng, Q., and Szlavnics, Z. 2000. Cultivar evaluation and
mega-environment investigation based on the GGE biplot. Crop Science, 40,
597–605.\n

Yan, W. and Kang, M.S. 2003. GGE Biplot Analysis: A Graphical Tool for Breeders,
Geneticists, and Agronomists. CRC Press, Boca Raton, FL. \n

Yan, W., Kang, M.S., Ma, B., Woods, S., and Cornelius, P.L. 2007. GGE Biplot vs.
AMMI analysis of genotype-by-environment data. Crop Science, 7, 641–653. \n

Yan, W. and Rajcan, I. 2002. Biplot analysis of sites and trait relations of
soybean in Ontario, Crop Science, 42, 11–20. \n

Yan, W. and Tinker, N.A. 2005. An integrated biplot system for displaying,
interpreting, and exploring genotype 9 environment interaction. Crop Science,
45, 1004–1016.  \n


# Session Info

```{r, echo = FALSE}
sessionInfo()
```

---
title: "How to use the geneticae Shiny App"
subtitle: "A web application for the Analysis of Multi Environment Agronomic Trials"
author: |
    | Julia Angelini
    | Marcos Prunello
    | Gerardo Cervigni
    |
    | Centro de Estudios Fotosintéticos y Bioquímicos
    | Universidad Nacional de Rosario
    | Rosario, Argentina
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{How to use the geneticae Shiny App}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(warning = F, collapse = TRUE, message = F, out.width = "60%")
```

<style type="text/css">

body{
text-align: justify}
</style>
  
The `geneticae` package can be used through [this](https://geneticae.shinyapps.io/geneticae-shiny-web-app/) Shiny app, making it available not only for R programmers. Source code is in [GitHub repo](https://github.com/jangelini/Geneticae-Shiny-Web-APP).



# Motivation

Although there are R packages which tackle different aspects of MET data analysis, there aren't any packages capable of performing all the steps that need to be considered. The geneticae package was created to gather in one place the most useful functions for this type of analysis and it also implements new methodology which can be found in recent literature. More importantly, geneticae is the first package to implement the robust AMMI model and new imputation methods not available before. In addition, there is no need to preprocess the data to use the `geneticae` package, as it the case of some previous packages which require a data frame or matrix containing genotype by environment means with the genotypes in rows and the environments in columns. In this package, data in long format is required. Genotypes, environments, repetitions (if any) and phenotypic traits of interest can be presented in any order and there is no restriction on columns names. Also, extra information that will not be used in the analysis may be present in the dataset. Finally, `geneticae` offers a wide variety of options to customize the biplots, which are part of the graphical output of these methods.

The goal of the Geneticae Shiny Web APP is to provide a graphical user interface for the R package, so that it can be used by breeders and analysts with no previous experience in R programming. It is an interactive, noncommercial and open source software, offering a free alternative to available commercial software to analize METs.

# Small example

If you are just getting started with Geneticae APP we recommend visiting and exploring the examples throughout the tutorial in the APP. Here we present a small example.

The dataset yan.winterwheat available as example dataset has information about the yield of 18 winter wheat varieties grown in nine environments in Ontario at 1993. The GGE biplot visually addresses many issues relative to genotype and test environment evaluation. The GGE Biplot tab allows to builds several GGE biplots views. The basic one is produced by default. If Which Won Where/What is indicate in plot type, and the other options are left by default the polygonal view of the GGE biplots is provides (Figure 1). This is an effective way to visualize the which-won-where pattern of MET data. Cultivars in the vertices of the polygon (Fun,Zav, Ena, Kat and Luc) are those with the longest vectors, in their respective directions, which is a measure of the ability to respond to environments. The vertex cultivars are, therefore, among the most responsive cultivars; all other cultivars are less responsive in their respective directions.

The dotted lines are perpendicular to the polygon sides and divide the biplot into mega-environments, each of which has a vertex cultivar, which is the one with the highest yield (phenotype) in all environments found in it. OA93 and KE93 are in the same sector, separated from the rest of the biplot by two perpendicular lines, and Zav is the highest-yielding cultivar in this sector. Fun is the highest-yielding cultivar in its sector, which contains seven environments, namely, EA93, BH93, HW93, ID93, WP93, NN93, and RN93. No environments fell in the sectors with Ena, Kat, and Luc as vertex cultivars. This indicates that these vertex cultivars were not the best in any of the test environments. Moreover, these cultivars were the poorest in some or all of the environments.


```{r, echo=F, fig.align='center', fig.cap='Figure: Polygon view of the GGE biplot, showing which cultivars presented highest yield in each environment. The scaling method used is symmetrical singular value partitioning (by default). The 78% of G + GE variability is explained by the first two multiplicative terms. Cultivars are shown in lowercase and environments in uppercase.'}
library(geneticae)
library(agridat)
data(yan.winterwheat)
GGE1 <- rSREGModel(yan.winterwheat, genotype = "gen", environment = "env", 
                 response = "yield")

rSREGPlot(GGE1, type = "Which Won Where/What", footnote = F,
        titles = F, axis_expand = 1.5)

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EM_GGE.R
\name{BiplotImputfun}
\alias{BiplotImputfun}
\title{Biplot Imputation Method}
\usage{
BiplotImputfun(X, precision = 0.01, max.iter = 1000, n_pc = 2)
}
\arguments{
\item{X}{A data frame or matrix with genotypes in rows and 
environments in columns.}

\item{precision}{(optional) Convergence threshold. The algorithm stops 
when the relative change in imputed values is less than this value. 
Default is 0.01.}

\item{max.iter}{(optional) Maximum number of iterations. Default is 1000.}

\item{n_pc}{Number of principal components to use for imputation. 
Default is 2.}
}
\value{
A list containing:
\itemize{
  \item \code{X_imputed}: The final matrix with missing values filled.
  \item \code{iteration}: Total number of iterations performed.
  \item \code{convergence}: Final relative change reached.
  \item \code{fitted}: The fitted values from the AMMI/Biplot model.
}
}
\description{
This function implements the Biplot imputation method as 
proposed by Yan (2013). It is an iterative algorithm that uses the singular 
value decomposition (SVD) to impute missing values in a genotype by 
environment matrix.
}
\references{
Yan, W. (2013). \emph{Biplot analysis of incomplete two-way data}. 
Crop Science, 53(1), 48-57. \doi{10.2135/cropsci2012.05.0301}

Arciniegas-Alarcón, S., García-Peña, M., Krzanowski, W., & Dias, C. T. S. 
(2014b). \emph{An alternative methodology for imputing missing data in 
trials with genotype-by-environment interaction: some new aspects}. 
Biometrical Letters, 51(2), 75-88. \doi{10.2478/bile-2014-0006}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rAMMIPlot.R
\name{rAMMIPlot}
\alias{rAMMIPlot}
\title{AMMI Biplots with \pkg{ggplot2}}
\usage{
rAMMIPlot(
  model_res,
  colGen = "gray47",
  colEnv = "darkred",
  sizeGen = 6,
  sizeEnv = 6,
  titles = TRUE,
  footnote = TRUE,
  axis_expand = 1.2,
  limits = TRUE,
  axes = TRUE,
  axislabels = TRUE
)
}
\arguments{
\item{model_res}{an object of class \code{rAMMI} generated by \code{\link{rAMMIModel}}.}

\item{colGen}{color for genotype labels. Defaults to `"gray47"`.}

\item{colEnv}{color for environment labels and vectors. Defaults to `"darkred"`.}

\item{sizeGen}{text size for genotype labels. Defaults to 6.}

\item{sizeEnv}{text size for environment labels. Defaults to 6.}

\item{titles}{logical. If `TRUE` (default), a title indicating the model 
type is added to the plot.}

\item{footnote}{logical. If `TRUE` (default), a caption with the percentage 
of explained GxE variation is added at the bottom.}

\item{axis_expand}{numeric value to expand the axis limits. Useful to prevent 
labels from being cut off. Defaults to 1.2.}

\item{limits}{logical. If `TRUE` (default), uses \code{coord_fixed()} to 
maintain a 1:1 aspect ratio between axes.}

\item{axes}{logical. If `TRUE` (default), draws dashed horizontal and 
vertical lines passing through the origin (0,0).}

\item{axislabels}{logical. If `TRUE` (default), includes axis titles with 
the percentage of variance explained by each principal component.}
}
\value{
A \pkg{ggplot2} object. This allows further customization using 
  standard ggplot layers (e.g., \code{+ theme_bw()}).
}
\description{
Generates a high-quality biplot (PC1 vs PC2) for classical or 
  robust AMMI models using the \pkg{ggplot2} framework.
}
\details{
the biplot is constructed using a scaling factor of 0.5 (symmetric scaling), 
which allows representing both genotypes and environments in the same 
algebraic space. Genotypes are displayed as points (text), and environments 
are represented as vectors from the origin.
}
\examples{
library(agridat)
data(yan.winterwheat)

# Classical AMMI
mod_ammi <- rAMMIModel(yan.winterwheat, genotype = "gen", 
                       environment = "env", response = "yield", type = "AMMI")

rAMMIPlot(mod_ammi,sizeGen=4,sizeEnv=4)

data(plrv)
mod_ammi_rep <- rAMMIModel(plrv, genotype="Genotype", environment="Locality", 
                        response="Yield", rep="Rep", type="AMMI")
rAMMIPlot(mod_ammi_rep,sizeGen=4,sizeEnv=4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rSREGPlot.R
\name{rSREGPlot}
\alias{rSREGPlot}
\title{GGE biplots with \pkg{ggplot2}}
\usage{
rSREGPlot(
  rSREGModel,
  type = "Biplot",
  d1 = 1,
  d2 = 2,
  selectedE = NA,
  selectedG = NA,
  selectedG1 = NA,
  selectedG2 = NA,
  colGen = "gray47",
  colEnv = "darkred",
  colSegment = "gray30",
  colHull = "gray30",
  sizeGen = 6,
  sizeEnv = 6,
  largeSize = 4.5,
  axis_expand = 1.2,
  axislabels = TRUE,
  axes = TRUE,
  limits = TRUE,
  titles = TRUE,
  footnote = TRUE
)
}
\arguments{
\item{rSREGModel}{An object of class \code{rSREGModel}.}

\item{type}{type of biplot to produce.
\itemize{
 \item \code{"Biplot"}: Basic biplot.
 \item \code{"Selected Environment"}: Ranking of cultivars based on
 their performance in any given environment.
 \item \code{"Selected Genotype"}: Ranking of environments based on the
 performance of any given cultivar.
 \item \code{"Relationship Among Environments"}.
 \item \code{"Comparison of Genotype"}.
 \item \code{"Which Won Where/What"}: Identifying the 'best'
 cultivar in each environment.
 \item \code{"Discrimination vs. representativeness"}: Evaluating the
 environments based on both discriminating ability and representativeness.
 \item \code{"Ranking Environments"}: Ranking environments with respect to
 the ideal environment.
 \item \code{"Mean vs. stability"}: Evaluating cultivars based on both average
 yield and stability.
 \item \code{"Ranking Genotypes"}: Ranking genotypes with respect to the
 ideal genotype.}}

\item{d1}{PCA component to plot on x axis. Defaults to 1.}

\item{d2}{PCA component to plot on y axis. Defaults to 2.}

\item{selectedE}{name of the environment to evaluate when `type="Selected
Environment"`.}

\item{selectedG}{name of the genotype to evaluate when `type="Selected
Genotype"`.}

\item{selectedG1, }{name of the genotype to compare to `selectedG2`  when
`type="Comparison of Genotype"`.}

\item{selectedG2, }{name of the genotype to compare to `selectedG1`  when
`type="Comparison of Genotype"`.}

\item{colGen}{genotype attributes colour. Defaults to `"gray47"`.}

\item{colEnv}{environment attributes colour. Defaults to `"darkred"`.}

\item{colSegment}{segment or circle lines colour. Defaults to `"gray30"`.}

\item{colHull}{hull colour when `type="Which Won Where/What"`. Defaults to
"gray30".}

\item{sizeGen}{genotype labels text size. Defaults to 4.}

\item{sizeEnv}{environment labels text size. Defaults to 4.}

\item{largeSize}{larger labels text size to use for two selected genotypes in
`type="Comparison of Genotype"`, and for the outermost genotypes in
`type="Which Won Where/What"`. Defaults to 4.5.}

\item{axis_expand}{multiplication factor to expand the axis limits by to
enable fitting of labels. Defaults to 1.2.}

\item{axislabels}{logical, if this argument is `TRUE` labels for axes are
included. Defaults to `TRUE`.}

\item{axes}{logical, if this argument is `TRUE` x and y axes going through the
origin are drawn. Defaults to `TRUE`.}

\item{limits}{logical, if this argument is `TRUE` the axes are re-scaled.
Defaults to `TRUE`.}

\item{titles}{logical, if this argument is `TRUE` a plot title is included.
Defaults to `TRUE`.}

\item{footnote}{logical, if this argument is `TRUE` a footnote is included.
Defaults to `TRUE`.}
}
\value{
A biplot of class \code{ggplot}
}
\description{
GGE biplots are used for visual examination of the relationships
 between test environments, genotypes, and genotype-by-environment
 interactions. `rSREGPlot()` produces a biplot as an object of class 'ggplot',
 using the output of the \code{\link{rSREGModel}} function.
 Several types of biplots are offered which focus on different aspects of the
 analysis. Customization options are also included. This function is a
 modification of the `rSREGPlot` function from the 
 \href{https://CRAN.R-project.org/package=GGEBiplots}{GGEBiplots package}.
}
\examples{
 library(geneticae)

 # Data without replication
 library(agridat)
 data(yan.winterwheat)
 GGE1 <- rSREGModel(yan.winterwheat)
 rSREGPlot(GGE1, sizeGen=4, sizeEnv=4)

 # Data with replication
 data(plrv)
 GGE2 <- rSREGModel(plrv, genotype = "Genotype", environment = "Locality",
                  response = "Yield", rep = "Rep")
 rSREGPlot(GGE2, sizeGen=4, sizeEnv=4)

}
\references{
Yan W, Kang M (2003). \emph{GGE Biplot Analysis: A Graphical Tool
 for Breeders, Geneticists, and Agronomists}. CRC Press.

Sam Dumble (2017). GGEBiplots: GGE Biplots with 'ggplot2'. R
 package version 0.1.1. \url{https://CRAN.R-project.org/package=GGEBiplots}
}
\keyword{Biplot}
\keyword{GGE}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rAMMIModel.R
\name{rAMMIModel}
\alias{rAMMIModel}
\title{Robust AMMI Model Fitting}
\usage{
rAMMIModel(
  Data,
  genotype = "gen",
  environment = "env",
  response = "Y",
  rep = NULL,
  Ncomp = 2,
  type = "AMMI"
)
}
\arguments{
\item{Data}{a dataframe with genotypes, environments, repetitions (if any)
and the phenotypic trait of interest. Other variables that will not be used
in the analysis can be included.}

\item{genotype}{column name containing genotypes. Defaults to `"gen"`.}

\item{environment}{column name containing environments. Defaults to `"env"`.}

\item{response}{column name containing the phenotypic trait of interest.
Defaults to `"Y"`.}

\item{rep}{column name containing replications. If this argument is `NULL`
(default), it is assumed that the data already contains means per genotype
in each environment. If provided, means are calculated automatically.}

\item{Ncomp}{number of principal components to retain for the interaction
part. Defaults to 2.}

\item{type}{method for fitting the AMMI model: `"AMMI"` (classical),
`"rAMMI"`, `"hAMMI"`, `"gAMMI"`, `"lAMMI"` or `"ppAMMI"` (robust variants).
Defaults to `"AMMI"`.}
}
\value{
A list of class \code{rAMMI} containing: \item{gen_scores}{Matrix of
  genotype scores (U * D).} \item{env_scores}{Matrix of environment loadings
  (V).} \item{eigenvalues}{Vector of singular values for the retained
  components.} \item{gen_labels}{Names of the genotypes.}
  \item{env_labels}{Names of the environments.} \item{Ncomp}{Number of
  principal components used.} \item{type}{The fitting method used.}
  \item{vartotal}{Total variance explained by the multiplicative terms.}
}
\description{
Fits a classical or robust Additive Main effects and
  Multiplicative Interaction (AMMI) model for genotype-by-environment data.
}
\details{
To overcome the problem of data contamination with outlying
observations, Rodrigues, Monteiro and Lourenco (2015) propose a robust AMMI
model based on the M-Huber estimator and robust SVD/PCA procedures.

The `type` argument allows choosing between several robust strategies:
\itemize{
  \item \bold{AMMI}: Classical AMMI model using Least Squares and standard SVD.
  \item \bold{rAMMI}: Uses the L1 norm instead of the L2 norm to compute a
    robust approximation to the SVD (via \pkg{pcaMethods}).
  \item \bold{hAMMI}: Uses the Hubert's approach (PcaHubert) combining
    projection-pursuit and robust covariance estimation.
  \item \bold{gAMMI}: Uses the Grid search algorithm for PCA (PcaGrid).
  \item \bold{lAMMI}: Performs PCA on the data projected onto a unit sphere (PcaLocantore).
  \item \bold{ppAMMI}: Uses projection-pursuit (PcaProj) to calculate robust
    eigenvalues and eigenvectors.
}
}
\examples{
library(agridat)
data(yan.winterwheat)

# Classical AMMI
mod_ammi <- rAMMIModel(yan.winterwheat, genotype = "gen",
                       environment = "env", response = "yield", type = "AMMI")

# Robust AMMI (using Hubert's method)
mod_rammi <- rAMMIModel(yan.winterwheat, genotype = "gen",
                        environment = "env", response = "yield", type = "hAMMI")


}
\references{
Rodrigues P.C., Monteiro A., Lourenco V.M. (2015). A robust AMMI
   model for the analysis of genotype-by-environment data. Bioinformatics 32,
   58-66.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{plrv}
\alias{plrv}
\title{Clones from the PLRV population}
\format{
Data frame with 504 observations and 6 variables (genotype,
  locality, repetition, weightPlant, weightPlot and yield).
}
\usage{
data(plrv)
}
\description{
resistance study to PLRV (Patato Leaf Roll Virus) causing leaf
curl. 28 genotypes were experimented at 6 locations in Peru. Each clone was
evaluated three times in each environment, and yield, plant weight and plot were
registered.
}
\examples{
library(geneticae)
data(plrv)
str(plrv)

}
\references{
Felipe de Mendiburu (2020). agricolae: Statistical Procedures for
 Agricultural Research. R package version 1.3-2.
 \url{https://CRAN.R-project.org/package=agricolae}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Eigenvector.R
\name{Eigenvectorfun}
\alias{Eigenvectorfun}
\title{Eigenvector Imputation Function (Internal)}
\usage{
Eigenvectorfun(X, f)
}
\arguments{
\item{X}{A matrix with missing values (NAs).}

\item{f}{Number of components (rank) to use for the reconstruction.}
}
\value{
A list with the number of iterations, convergence status, 
final rank used, and the imputed matrix.
}
\description{
Internal function for GxE imputation using the Krzanowski (1988) eigenvector 
approach with a leave-one-out strategy.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EM_AMMI.R
\name{EM.AMMI}
\alias{EM.AMMI}
\title{EM-AMMI imputation method}
\usage{
EM.AMMI(
  X,
  PC.nb = 1,
  initial.values = NA,
  precision = 0.01,
  max.iter = 1000,
  change.factor = 1,
  simplified.model = FALSE
)
}
\arguments{
\item{X}{a data frame or matrix with genotypes in rows and
environments in columns when there are no replications of the
experiment.}

\item{PC.nb}{the number of principal components in the AMMI model that will be
used; default value is 1. For PC.nb=0 only main effects are used to
estimate cells in the data table (the interaction is ignored). The number of
principal components must not be greater than min(number of rows in the X
table, number of columns in the X table)–2.}

\item{initial.values}{(optional) initial values of missing cells. It
can be a single value, which then will be used for all empty cells, or a
vector of length equal to the number of missing cells (starting from the
missing values in the first column). If omitted, initial values will be
obtained by the main effects from the corresponding model, that is, by
grand mean of observed data increased (or decreased) by row and column
main effects.}

\item{precision}{(optional) algorithm converges if the maximal change in
the values of the missing cells in two subsequent steps is not greater than
this value (the default is 0.01).}

\item{max.iter}{(optional) a maximum permissible number of iterations (that
is, number of repeats of the algorithm’s steps 2 through 5); default
value is 1000.}

\item{change.factor}{(optional) introduced by analogy to step size in gradient
descent method, this parameter that can shorten the time of executing the
algorithm by decreasing the number of iterations. The change.factor=1
(default) defines that the previous approximation is changed with the new
values of missing cells (standard EM-AMMI algorithm). However, when
change.factor<1, then the new approximations are computed and the values of
missing cells are changed in the direction of this new approximation but the
change is smaller. It could be useful if the changes are cyclic and thus
convergence could not be reached. Usually, this argument should not affect
the final outcome (that is, the imputed values) as compared to the default
value of change.factor=1.}

\item{simplified.model}{the AMMI model contains the general mean, effects of
rows, columns and interaction terms. So the EM-AMMI algorithm in step 2
calculates the current effects of rows and columns; these effects change
from iteration to iteration because the empty (at the outset) cells in each
iteration are filled with different values. In step 3 EM-AMMI uses those
effects to re-estimate cells marked as missed (as default,
simplified.model=FALSE). It is, however, possible that this procedure will
not converge. Thus the user is offered a simplified EM-AMMI procedure that
calculates the general mean and effects of rows and columns only in the
first iteration and in next iterations uses these values
(simplified.model=TRUE). In this simplified procedure the initial values
affect the outcome (whilst EM-AMMI results usually do not depend on initial
values). For the simplified procedure the number of iterations to
convergence is usually smaller and, furthermore, convergence will be reached
even in some cases where the regular procedure fails. If the regular
procedure does not converge for the standard initial values (see the
description of the argument initial.values), the simplified model can be
used to determine a better set of initial values.}
}
\value{
A list containing:
 \itemize{
   \item  X: the imputed matrix (filled in with the missing values estimated
   by the EM-AMMI procedure);
   \item PC.SS: the sum of squares representing variation explained
   by the principal components (the squares of eigenvalues of singular value
   decomposition);
   \item iteration: the final number of iterations;
   \item precision.final: the maximum change of the estimated values for
   missing cells in the last step of iteration (the precision of convergence)
   . If the algorithm converged, this value is slightly smaller than the
   argument precision;
   \item PC.nb.final: a number of principal components that were eventually
   used by the EM.AMMI() function. The function checks if there are too many
   missing cells to unambiguously compute the parameters by the SVD
   decomposition (Gauch and Zobel, 1990). In that case the final number of
   principal components used is smaller than that which was passed on to the
   function through the argument PC.nb;
   \item convergence: the value TRUE means that the algorithm converged in
   the last iteration.
 }
}
\description{
This function was written by Paderewski (2013) and allow imputing
 missing values by the EM-AMMI algorithm
}
\references{
Paderewski, J. (2013). \emph{An R function for imputation of
 missing cells in two-way initial.values), the simplified model can be used
 to determine a better set of initial values. data sets by EM-AMMI
 algorithm.}. Communications in Biometry and Crop Science 8, 60–69.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rSREGModel.R
\name{rSREGModel}
\alias{rSREGModel}
\title{Site Regression model}
\usage{
rSREGModel(
  Data,
  genotype = "gen",
  environment = "env",
  response = "yield",
  rep = NULL,
  model = "SREG",
  SVP = "symmetrical"
)
}
\arguments{
\item{Data}{dataframe with genotypes, environments, repetitions (if any) and
the phenotypic trait of interest. Additional variables that will not be used
in the model may be present in the data.}

\item{genotype}{column name for genotypes.}

\item{environment}{column name for environments.}

\item{response}{column name for the phenotypic trait.}

\item{rep}{column name for replications. If this argument is NULL, there are
no replications in the data. Defaults to NULL.}

\item{model}{method for fitting the SREG model: `"SREG"`,`"CovSREG"`,`"hSREG"`
or `"ppSREG"` (see References). Defaults to `"SREG"`.}

\item{SVP}{method for singular value partitioning. Either `"row"`, `"column"`,
or `"symmetrical"`. Defaults to `"symmetrical"`.}
}
\value{
A list of class \code{GGE_Model} containing: \item{model}{SREG model
 version.} \item{coordgenotype}{plotting coordinates for each genotype in
 every component.} \item{coordenviroment}{plotting coordinates for each
 environment in every component.} \item{eigenvalues}{vector of eigenvalues
 for each component.} \item{vartotal}{overall variance.}
 \item{varexpl}{percentage of variance explained by each component.}
 \item{labelgen}{genotype names.} \item{labelenv}{environment names.}
 \item{axes}{axis labels.} \item{Data}{scaled and centered input data.}
 \item{SVP}{name of SVP method.}

A biplot of class \code{ggplot}
}
\description{
The Site Regression model (also called genotype +
 genotype-by-environment (GGE) model) is a powerful tool for effective
 analysis and interpretation of data from multi-environment trials in
 breeding programs. There are different functions in R to fit the SREG model,
 however, this function has the following improvements: \itemize{ \item
 Includes recently published robust versions of the SREG model (Angelini et
 al., 2022). \item It can be used for data from trials with repetitions
 (there is no need to calculate means beforehand). \item Other variables not
 used in the analysis can be present in the dataset.}
}
\details{
A linear model by robust regression using an M estimator proposed by
 Huber (1964, 1973) fitted by iterated re-weighted least squares, in
 combination with three robust SVD/PCA procedures, resulted in a total of
 three robust SREG alternatives. The robust SVD/PCA considered were:
 \itemize{ \item CovSREG: robust PCA that is obtained by replacing the
 classical estimates of location and covariance by their robust analogues
 using Minimum Regularized Covariance Determinant (MRCD) approach; \item
 hSREG: robust PCA method that tries to combine the advantages of both
 approaches, PCA based on a robust covariance matrix and based on projection
 pursuit; \item ppSREG: robust PCA that uses the projection pursuit and
 directly calculates the robust estimates of the eigenvalues and eigenvectors
 without going through robust covariance estimation. It is a very attractive
 method for bigdata situations, which are very common in METs (a few
 genotypes tested in a large number of environments), as the principal
 components can be calculated sequentially. }
}
\examples{
 library(geneticae)

 # Data without replication
 library(agridat)
 data(yan.winterwheat)
 GGE1 <- rSREGModel(yan.winterwheat, genotype="gen", environment="env", response="yield")

 # Data with replication
 data(plrv)
 GGE2 <- rSREGModel(plrv, genotype = "Genotype", environment = "Locality",
                  response = "Yield", rep = "Rep")

}
\references{
Julia Angelini, Gabriela Faviere, Eugenia Bortolotto, Gerardo
 Domingo Lucio Cervigni & Marta Beatriz Quaglino (2022) Handling outliers in
 multi-environment trial data analysis: in the direction of robust SREG
 model, Journal of Crop Improvement, \doi{10.1080/15427528.2022.2051217}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GabrielEigen.R
\name{Gabriel.Calinski}
\alias{Gabriel.Calinski}
\title{GabrielEigen imputation method}
\usage{
Gabriel.Calinski(X)
}
\arguments{
\item{X}{a data frame or matrix with genotypes in rows and
environments in columns when there are no replications of the
experiment.}
}
\value{
A list containing:
  \itemize{
    \item NumeroIterGabriel: final number of iterations;
    \item CritConvergGabriel: maximum change of the estimated values for
    missing cells in the last step of iteration (precision of convergence);
    \item GabrielImput: imputed matrix (filled in with missing values
    estimated by the GabrielEigein procedure).
  }
}
\description{
GabrielEigen imputation method
}
\references{
Arciniegas-Alarcón S., García-Peña M., Dias C.T.S., Krzanowski
  W.J. (2010). \emph{An alternative methodology for imputing missing data in
  trials with genotype-by-environment interaction}. Biometrical Letters 47,
  1–14.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute.R
\name{imputation}
\alias{imputation}
\title{Imputation of missing cells in two-way data sets}
\usage{
imputation(
  Data,
  genotype = "gen",
  environment = "env",
  response = "yield",
  rep = NULL,
  type = "EM-AMMI",
  nPC = 2,
  initial.values = NA,
  precision = 0.01,
  maxiter = 1000,
  change.factor = 1,
  simplified.model = FALSE,
  scale = TRUE,
  method = "EM",
  row.w = NULL,
  coeff.ridge = 1,
  seed = NULL,
  nb.init = 1,
  Winf = 0.8,
  Wsup = 1
)
}
\arguments{
\item{Data}{dataframe containing genotypes, environments, repetitions (if any)
and the phenotypic trait of interest. Other variables that will not be used
in the analysis can be present.}

\item{genotype}{column name containing genotypes.}

\item{environment}{column name containing environments.}

\item{response}{column name containing the phenotypic trait.}

\item{rep}{column name containing replications. If this argument is NULL,
there are no replications available in the data. Defaults to NULL.}

\item{type}{imputation method. Either "EM-AMMI", "EM-GGE", "EM-SREG", "EM-bSREG",
"Gabriel", "Eigenvector", "WGabriel", "EM-PCA". Defaults to "EM-AMMI".}

\item{nPC}{number of components used to predict the missing values.
Default to 2.}

\item{initial.values}{initial values of the missing cells. It can be a single
value or a vector of length equal to the number of missing cells.}

\item{precision}{threshold for assessing convergence.}

\item{maxiter}{maximum number of iteration for the algorithm.}

\item{change.factor}{When `change.factor` is equal to 1, the previous
approximation is changed with the new values (standard EM). Smaller values
can help convergence if changes are cyclic.}

\item{simplified.model}{logical. If TRUE, calculates effects only in the first 
iteration to speed up convergence or help in cases where the regular 
procedure fails.}

\item{scale}{boolean. By default TRUE for "EM-PCA".}

\item{method}{"Regularized" or "EM" for "EM-PCA".}

\item{row.w}{row weights for "EM-PCA".}

\item{coeff.ridge}{ridge coefficient for "EM-PCA".}

\item{seed}{integer for random initialization in "EM-PCA".}

\item{nb.init}{number of random initializations for "EM-PCA".}

\item{Winf}{lower weight for WGabriel.}

\item{Wsup}{upper weight for WGabriel.}
}
\value{
A matrix of the imputed data.
}
\description{
Missing values are not allowed by the AMMI, GGE or SREG methods. This function
provides several methods to impute missing observations in data from
multi-environment trials and to subsequently adjust the mentioned methods.
}
\details{
Often, multi-environment experiments are unbalanced because several genotypes
are not tested in some environments. Several methodologies have been proposed
in order to solve this lack of balance caused by missing values, some of which
are included in this function:

\itemize{
\item EM-AMMI: an iterative scheme built round the above procedure is used to
obtain AMMI imputations from the EM algorithm. The additive parameters are
initially set by computing the grand mean, genotype means and environment
means obtained from the observed data. The residuals for the observed cells
are initialized as the cell mean minus the genotype mean minus the environment
mean plus the grand mean, and interactions for the missing positions are
initially set to zero. The initial multiplicative parameters are obtained from
the SVD of this matrix of residuals, and the missing values are filled by the
appropriate AMMI estimates. In subsequent iterations, the usual AMMI procedure
is applied to the completed matrix and the missing values are updated by the
corresponding AMMI estimates. The arguments used for this method
are:initial.values, precision, maxiter, change.factor and simplified.model

\item EM-GGE: Iterative SVD-based imputation focusing on G+GE.

\item EM-SREG: Iterative algorithm using the Sites Regression model. Supports variants 
 like standard SVD and Bayesian PCA (EM-bSREG). 

\item Gabriel: combines regression and lower-rank approximation using SVD.
This method initially replaces the missing cells by arbitrary values, and
subsequently the imputations are refined through an iterative scheme that
defines a partition of the matrix for each missing value in turn and uses a
linear regression of columns (or rows) to obtain the new imputation. The
arguments used for this method is only the dataframe.

\item WGabriel: is a a modification of Gabriel method that uses weights chosen
by cross-validation. The arguments used for this method are Winf and Wsup.

\item EM-PCA: impute the missing entries of a mixed data using the iterative
PCA algorithm. The algorithm first consists imputing missing values with
initial values. The second step of the iterative PCA algorithm is to perform
PCA on the completed dataset to estimate the parameters. Then, it imputes the
missing values with the reconstruction formulae of order nPC (the fitted
matrix computed with nPC components for the scores and loadings). These steps
of estimation of the parameters via PCA and imputation of the missing values
using the fitted matrix are iterate until convergence. The arguments used for
this methods are: nPC, scale, method, row.w, coeff.ridge, precision, seed,
nb.init and maxiter

}
}
\examples{
library(geneticae)
# Data without replications
library(agridat)
data(yan.winterwheat)

# generating missing values
yan.winterwheat[1,3]<-NA
yan.winterwheat[3,3]<-NA
yan.winterwheat[2,3]<-NA

imputation(yan.winterwheat, genotype = "gen", environment = "env",
           response = "yield", type = "EM-AMMI")

# Data with replications
data(plrv)
head(plrv)
plrv$Yield[plrv$Locality == "Ayac" & plrv$Rep \%in\% c(1, 2, 3) & plrv$Genotype == '102.18'] <- NA

imputation(plrv, nPC = 2,genotype = "Genotype", environment = "Locality", 
           response = "Yield", rep ='Rep', type = "EM-AMMI")
           
imputation(plrv, genotype = "Genotype", environment = "Locality", 
           response = "Yield", rep ='Rep', type = "EM-SREG")

}
\references{
Paderewski, J. (2013). \emph{An R function for imputation of missing
 cells in two-way data sets by EM-AMMI algorithm}. Communications in Biometry
 and Crop Science 8, 60–69.

Yan, W. (2013). \emph{Biplot analysis of incomplete two-way data}. 
Crop Science, 53(1), 48-57. \doi{10.2135/cropsci2012.05.0301}

Arciniegas-Alarcón, S., García-Peña, M., Krzanowski, W., & Dias, C. T. S. 
(2014b). \emph{An alternative methodology for imputing missing data in 
trials with genotype-by-environment interaction: some new aspects}. 
Biometrical Letters, 51(2), 75-88. \doi{10.2478/bile-2014-0006}

Angelini, J., Cervigni, G. D. L., & Quaglino, M. B. (2024). \emph{New 
imputation methodologies for genotype-by-environment data: an extensive 
study of properties of estimators}. Euphytica, 220(6), 92. 
\doi{10.1007/s10681-024-03344-z}

Julie Josse, Francois Husson (2016). missMDA: A Package for
 Handling Missing Values in Multivariate Data Analysis. Journal of
 Statistical Software 70, 1-31.

Arciniegas-Alarcón S., García-Peña M., Dias C.T.S., Krzanowski
 W.J. (2010). \emph{An alternative methodology for imputing missing data in
 trials with genotype-by-environment interaction}. Biometrical Letters 47,
 1–14.

Arciniegas-Alarcón S., García-Peña M., Krzanowski W.J., Dias
 C.T.S. (2014). \emph{An alternative methodology for imputing missing data in
 trials with genotype-by-environment interaction: some new aspects.}
 Biometrical Letters 51, 75-88.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EM_SREG.R
\name{EM.SREG}
\alias{EM.SREG}
\title{EM-SREG Imputation Method}
\usage{
EM.SREG(
  X,
  PC.nb = 1,
  initial.values = NA,
  precision = 0.01,
  max.iter = 1000,
  change.factor = 1,
  simplified.model = FALSE,
  type = c("EM-SREG", "EM-bSREG")
)
}
\arguments{
\item{X}{A data frame or matrix with genotypes in rows and environments in columns.}

\item{PC.nb}{Number of principal components to be used. Default is 1.}

\item{initial.values}{(optional) Initial values for missing cells. If NA, 
initial values are obtained from column means (environment effects).}

\item{precision}{Convergence threshold. Default is 0.01.}

\item{max.iter}{Maximum number of iterations. Default is 1000.}

\item{change.factor}{Step size for updating missing values (standard is 1).}

\item{simplified.model}{Logical. If TRUE, effects are only calculated in the first iteration.}

\item{type}{Method type: "EM-SREG" (Standard), "EM-bSREG" (Bayesian).}
}
\value{
A list containing:
\itemize{
  \item \code{X}: The imputed matrix.
  \item \code{iter}: The number of iterations until convergence.
}
}
\description{
Iterative algorithm to impute missing values in two-way tables 
  using the Sites Regression (SREG) model. It supports several variants 
  including standard SVD and Bayesian PCA.
}
\references{
Angelini, J., Cervigni, G. D. L., & Quaglino, M. B. (2024). \emph{New 
imputation methodologies for genotype-by-environment data: an extensive 
study of properties of estimators}. Euphytica, 220(6), 92. 
\doi{10.1007/s10681-024-03344-z}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/W_GabrielEigen.R
\name{WGabriel}
\alias{WGabriel}
\title{Weighted GabrielEigen imputation method}
\usage{
WGabriel(DBmiss, Winf, Wsup)
}
\arguments{
\item{DBmiss}{a data frame or matrix that contains the genotypes in the rows
and the environments in the columns when there are no replications of the
experiment.}

\item{Winf}{inferior weight}

\item{Wsup}{superior weight}
}
\value{
A list containing:
 \itemize{
   \item Peso: weight that provides the best predictive difference;
   \item NumeroIterWGabriel: the final number of iterations;
   \item CritConvergWGabriel: the maximum change of the estimated
   values for missing cells in the last step of iteration (the precision of
   convergence);
   \item GabrielWImput: the imputed matrix (filled in with the
   missing values estimated by the Weighted GabrielEigen procedure).
 }
}
\description{
Weighted GabrielEigen imputation method
}
\references{
Arciniegas-Alarcón S., García-Peña M., Krzanowski W.J., Dias
 C.T.S. (2014). An alternative methodology for imputing missing data in
 trials with genotype-by-environment interaction: some new aspects.
 Biometrical Letters 51, 75-88.
}
\keyword{internal}
